# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
The user specific resources

This module specifies all endpoints to manage user accounts
in the redis database via REST API calls.

TODO: Implement POST full permission creation
      Implement PUT to modify existing users
"""
from flask_restful import reqparse
from flask_restful_swagger_2 import swagger
from flask import jsonify, make_response, g
from actinia_api.swagger2.actinia_core.apidocs import user_api_key
from actinia_api.swagger2.actinia_core.schemas.user_api_key import (
    TokenResponseModel,
)

from actinia_core.rest.base.base_login import LoginBase
from actinia_core.core.common.app import auth
from actinia_core.core.common.api_logger import log_api_call
from actinia_core.rest.base.endpoint_config import (
    check_endpoint,
    endpoint_decorator,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = (
    "Copyright 2016-2021, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


# Create a temporal module where, order, column parser
expiration_time_parser = reqparse.RequestParser()
expiration_time_parser.add_argument(
    "expiration_time",
    required=False,
    type=int,
    location="args",
    help="The expiration time in seconds for the generated token",
)


class APIKeyCreationResource(LoginBase):
    """Get an API key that has no expiration time"""

    def __init__(self):
        LoginBase.__init__(self)

    @endpoint_decorator()
    @swagger.doc(check_endpoint("get", user_api_key.apikey_get_doc))
    def get(self):
        """Create an API key for permanent authentication."""

        try:
            return make_response(
                jsonify(
                    TokenResponseModel(
                        status="success",
                        token=g.user.generate_api_key(),
                        message="API key successfully generated",
                    )
                )
            )
        except Exception:
            return make_response(
                jsonify(
                    TokenResponseModel(
                        status="error",
                        token="",
                        message="Error while generating API key",
                    )
                ),
                400,
            )


class TokenCreationResource(LoginBase):
    """Get an authorization token"""

    decorators = [log_api_call, auth.login_required]

    def __init__(self):
        LoginBase.__init__(self)

    @endpoint_decorator()
    @swagger.doc(check_endpoint("get", user_api_key.token_get_doc))
    def get(self):
        """Create an authentication token."""

        args = expiration_time_parser.parse_args()
        expiration = 86400

        try:
            if "expiration_time" in args:
                if args["expiration_time"]:
                    expiration = args["expiration_time"]

                    if expiration > 365 * 86400:
                        raise Exception(
                            "Expiration time is to large. Maximum is 365 days."
                        )

            return make_response(
                jsonify(
                    TokenResponseModel(
                        status="success",
                        token=g.user.generate_auth_token(
                            expiration=expiration
                        ),
                        message="Token successfully generated with "
                        "an expiration time of %i seconds" % expiration,
                    )
                )
            )
        except Exception as e:
            return make_response(
                jsonify(
                    TokenResponseModel(
                        status="error",
                        token="",
                        message="Error while generating token: %s" % str(e),
                    )
                ),
                400,
            )



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Raster layer resources
"""
import os
from actinia_core.processing.actinia_processing.ephemeral.persistent_processing import (
    PersistentProcessing,
)
from actinia_core.core.common.exceptions import AsyncProcessError

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika, Guido Riembauer, Anika Weinmann"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class PersistentRasterDeleter(PersistentProcessing):
    def __init__(self, *args):
        PersistentProcessing.__init__(self, *args)

    def _execute(self):
        """Delete a specific raster layer

        Use the original mapset for processing
        """
        self._setup()

        raster_name = self.map_name
        self.required_mapsets.append(self.target_mapset_name)

        pc = {}
        pc["1"] = {
            "module": "g.remove",
            "inputs": {"type": "raster", "name": raster_name},
            "flags": "f",
        }

        self.skip_region_check = True
        process_list = self._validate_process_chain(
            process_chain=pc, skip_permission_check=True
        )
        self._check_lock_target_mapset()
        self._create_temp_database(self.required_mapsets)
        self._create_grass_environment(
            grass_data_base=self.temp_grass_data_base,
            mapset_name=self.target_mapset_name,
        )

        self._execute_process_list(process_list)

        if "WARNING: No data base element files found" in "\n".join(
            self.module_output_log[0]["stderr"]
        ):
            raise AsyncProcessError(
                "Raster layer <%s> not found" % raster_name
            )

        self.finish_message = (
            "Raster layer <%s> successfully removed." % raster_name
        )


class PersistentRasterCreator(PersistentProcessing):
    def __init__(self, *args):
        PersistentProcessing.__init__(self, *args)

    def _execute(self):
        """Create a specific raster layer

        This approach is complex, since the raster generation is performed in a
        localc temporary mapset that is later merged into the target mapset.
        Workflow:

        1. Check the process chain
        2. Lock the temp and target mapsets
        3. Setup GRASS and create the temporary mapset
        4. Execute g.list of the first process chain to check if the target
           raster exists
        5. If the target raster does not exists then run r.import
        6. Copy the local temporary mapset to the storage and merge it into the
           target mapset
        """
        self._setup()

        raster_name = self.map_name
        self.required_mapsets.append(self.target_mapset_name)

        pc_1 = {}
        pc_1["1"] = {
            "module": "g.list",
            "inputs": {
                "type": "raster",
                "pattern": raster_name,
                "mapset": self.target_mapset_name,
            },
        }
        # Check the first process chain
        pc_1 = self._validate_process_chain(
            skip_permission_check=True, process_chain=pc_1
        )

        pc_2 = {}
        pc_2["1"] = {
            "module": "r.import",
            "inputs": {"input": self.rdc.request_data, "output": raster_name},
        }
        # Check the second process chain
        pc_2 = self._validate_process_chain(
            skip_permission_check=True, process_chain=pc_2
        )

        self._check_lock_target_mapset()
        self._lock_temp_mapset()
        self._create_temporary_grass_environment()
        self._execute_process_list(pc_1)

        # check if raster exists
        raster_list = self.module_output_log[0]["stdout"].split("\n")

        if len(raster_list[0]) > 0:
            try:
                os.remove(self.rdc.request_data)
            except Exception:
                pass
            raise AsyncProcessError(
                "Raster layer <%s> exists. Please rename it or delete the old "
                "raster layer" % raster_name
            )

        self._execute_process_list(pc_2)
        self._copy_merge_tmp_mapset_to_target_mapset()

        # Delete imported file
        msg = ""
        try:
            os.remove(self.rdc.request_data)
        except Exception:
            msg = " WARNING: Uploaded file can not be removed."

        self.finish_message = (
            f"Raster layer <{raster_name}> successfully " f"imported.{msg}"
        )



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2021 mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Process Chain Monitoring
"""
import matplotlib.pyplot as plt
import numpy as np
import os
import pickle
from tempfile import NamedTemporaryFile
from flask import jsonify, make_response, Response
from flask_restful_swagger_2 import swagger
from actinia_api.swagger2.actinia_core.apidocs import process_chain_monitoring
from actinia_api.swagger2.actinia_core.schemas.process_chain_monitoring import (
    MapsetSizeResponseModel,
    MaxMapsetSizeResponseModel,
)

from actinia_core.rest.base.endpoint_config import (
    check_endpoint,
    endpoint_decorator,
)
from actinia_core.rest.resource_management import ResourceManager
from actinia_core.models.response_models import SimpleResponseModel

__license__ = "GPLv3"
__author__ = "Anika Weinmann, Carmen Tawalika"
__copyright__ = "Copyright 2021, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis"


def create_scatter_plot(x, y, xlabel, ylabel, title):
    plt.clf()
    plt.scatter(x, y, s=(np.pi * 5), c=(1, 0, 0))
    plt.title(title, fontsize=14)
    plt.xlabel(xlabel, fontsize=12)
    plt.ylabel(ylabel, fontsize=12)
    y_up = (max(y) + 9) // 10 * 10
    plt.ylim(0, y_up)
    step = y_up / 10
    plt.xticks(np.arange(1, len(y) + 1, step=step))
    with NamedTemporaryFile(suffix=".png", mode="wb", delete=False) as f:
        plt.savefig(f)
    return f.name


def compute_mapset_size_diffs(mapset_sizes):
    diffs = [None] * len(mapset_sizes)
    # TODO not correct cause an empty mapset has also a size!
    diffs[0] = mapset_sizes[0]
    for i in range(1, len(mapset_sizes)):
        diffs[i] = mapset_sizes[i] - mapset_sizes[i - 1]
    return diffs


class MapsetSizeResource(ResourceManager):
    """
    This class returns the mapset sizes of a resource
    """

    def __init__(self):
        # Configuration
        ResourceManager.__init__(self)

    @endpoint_decorator()
    @swagger.doc(
        check_endpoint("get", process_chain_monitoring.mapset_size_get_doc)
    )
    def get(self, user_id, resource_id):
        """Get the sizes of mapset of a resource."""

        ret = self.check_permissions(user_id=user_id)
        if ret:
            return ret

        response_data = self.resource_logger.get(user_id, resource_id)

        if response_data is not None:
            http_code, pc_response_model = pickle.loads(response_data)

            pc_status = pc_response_model["status"]
            if pc_status in ["accepted", "running"]:
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error",
                            message="Resource is not ready it is %s"
                            % pc_status,
                        )
                    ),
                    400,
                )

            mapset_sizes = [
                proc["mapset_size"]
                for proc in pc_response_model["process_log"]
            ]

            return make_response(
                jsonify(
                    MapsetSizeResponseModel(
                        status="success", mapset_sizes=mapset_sizes
                    )
                ),
                http_code,
            )
        else:
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error", message="Resource does not exist"
                    )
                ),
                400,
            )


class MapsetSizeDiffResource(ResourceManager):
    """
    This class returns the step-by-step mapset size differences of a resource
    """

    def __init__(self):
        # Configuration
        ResourceManager.__init__(self)

    @endpoint_decorator()
    @swagger.doc(
        check_endpoint("get", process_chain_monitoring.mapset_sizediff_get_doc)
    )
    def get(self, user_id, resource_id):
        """Get the step-by-step mapset size differences of a resource."""

        ret = self.check_permissions(user_id=user_id)
        if ret:
            return ret

        response_data = self.resource_logger.get(user_id, resource_id)

        if response_data is not None:
            http_code, pc_response_model = pickle.loads(response_data)

            pc_status = pc_response_model["status"]
            if pc_status in ["accepted", "running"]:
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error",
                            message="Resource is not ready it is %s"
                            % pc_status,
                        )
                    ),
                    400,
                )

            mapset_sizes = [
                proc["mapset_size"]
                for proc in pc_response_model["process_log"]
            ]
            diffs = compute_mapset_size_diffs(mapset_sizes)

            return make_response(
                jsonify(
                    MapsetSizeResponseModel(
                        status="success", mapset_sizes=diffs
                    )
                ),
                http_code,
            )
        else:
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error", message="Resource does not exist"
                    )
                ),
                400,
            )


class MaxMapsetSizeResource(ResourceManager):
    """
    This class returns the maximum mapset size of a resource
    """

    def __init__(self):
        # Configuration
        ResourceManager.__init__(self)

    @endpoint_decorator()
    @swagger.doc(
        check_endpoint("get", process_chain_monitoring.mapset_maxsize_get_doc)
    )
    def get(self, user_id, resource_id):
        """Get the maximum size of mapset of a resource."""

        ret = self.check_permissions(user_id=user_id)
        if ret:
            return ret

        response_data = self.resource_logger.get(user_id, resource_id)

        if response_data is not None:
            http_code, pc_response_model = pickle.loads(response_data)

            pc_status = pc_response_model["status"]
            if pc_status in ["accepted", "running"]:
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error",
                            message="Resource is not ready it is %s"
                            % pc_status,
                        )
                    ),
                    400,
                )

            mapset_sizes = [
                proc["mapset_size"]
                for proc in pc_response_model["process_log"]
            ]
            max_mapset_size = max(mapset_sizes)

            return make_response(
                jsonify(
                    MaxMapsetSizeResponseModel(
                        status="success", max_mapset_size=max_mapset_size
                    )
                ),
                http_code,
            )
        else:
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error", message="Resource does not exist"
                    )
                ),
                400,
            )


class MapsetSizeRenderResource(ResourceManager):
    """
    This class renders the mapset sizes of a resource
    """

    def __init__(self):
        # Configuration
        ResourceManager.__init__(self)

    @endpoint_decorator()
    @swagger.doc(
        check_endpoint(
            "get", process_chain_monitoring.mapset_render_sizes_get_doc
        )
    )
    def get(self, user_id, resource_id):
        """Render the mapset sizes of a resource."""

        ret = self.check_permissions(user_id=user_id)
        if ret:
            return ret

        response_data = self.resource_logger.get(user_id, resource_id)

        if response_data is not None:
            http_code, pc_response_model = pickle.loads(response_data)

            pc_status = pc_response_model["status"]
            if pc_status in ["accepted", "running"]:
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error",
                            message="Resource is not ready it is %s"
                            % pc_status,
                        )
                    ),
                    400,
                )

            mapset_sizes = [
                proc["mapset_size"]
                for proc in pc_response_model["process_log"]
            ]

            y = np.array(mapset_sizes)
            x = np.array(list(range(1, len(mapset_sizes) + 1)))
            unit = "bytes"
            for new_unit in ["KB", "MB", "GB", "TB"]:
                if max(y) > 1024.0:
                    y = y / 1024.0
                    unit = new_unit
                    print(new_unit)
                else:
                    break

            # create png
            result_file = create_scatter_plot(
                x,
                y,
                "process chain steps",
                "mapset size [%s]" % unit,
                "Mapset sizes of the resource\n%s" % resource_id,
            )

            if result_file:
                if os.path.isfile(result_file):
                    image = open(result_file, "rb").read()
                    os.remove(result_file)
                    return Response(image, mimetype="image/png")
        else:
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error", message="Resource does not exist"
                    )
                ),
                400,
            )


class MapsetSizeDiffRenderResource(ResourceManager):
    """
    This class renders the step-by-step mapset size differences of a resource
    """

    def __init__(self):
        # Configuration
        ResourceManager.__init__(self)

    @endpoint_decorator()
    @swagger.doc(
        check_endpoint(
            "get", process_chain_monitoring.mapset_render_sizediff_get_doc
        )
    )
    def get(self, user_id, resource_id):
        """Render the step-by-step mapset size differences of a resource."""

        ret = self.check_permissions(user_id=user_id)
        if ret:
            return ret

        response_data = self.resource_logger.get(user_id, resource_id)

        if response_data is not None:
            http_code, pc_response_model = pickle.loads(response_data)

            pc_status = pc_response_model["status"]
            if pc_status in ["accepted", "running"]:
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error",
                            message="Resource is not ready, it is %s"
                            % pc_status,
                        )
                    ),
                    400,
                )

            mapset_sizes = [
                proc["mapset_size"]
                for proc in pc_response_model["process_log"]
            ]
            diffs = compute_mapset_size_diffs(mapset_sizes)

            y = np.array(diffs)
            x = np.array(list(range(1, len(mapset_sizes) + 1)))
            unit = "bytes"
            for new_unit in ["KB", "MB", "GB", "TB"]:
                if max(y) > 1024.0:
                    y = y / 1024.0
                    unit = new_unit
                    print(new_unit)
                else:
                    break

            # create png
            result_file = create_scatter_plot(
                x,
                y,
                "process chain steps",
                "mapset size [%s]" % unit,
                "Step-by-step mapset size differences of the resource\n%s"
                % resource_id,
            )

            if result_file:
                if os.path.isfile(result_file):
                    image = open(result_file, "rb").read()
                    os.remove(result_file)
                    return Response(image, mimetype="image/png")
        else:
            return make_response(
                jsonify(
                    SimpleResponseModel(
                        status="error", message="Resource does not exist"
                    )
                ),
                400,
            )



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Resource logger and management interface
"""
import pickle
from .redis_resources import RedisResourceInterface
from .redis_fluentd_logger_base import RedisFluentLoggerBase

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika, Anika Weinmann"
__copyright__ = (
    "Copyright 2016-present, Sören Gebbert and mundialis GmbH & Co. KG"
)


class ResourceLogger(RedisFluentLoggerBase):
    """Write, update, receive and delete entries in the resource database"""

    def __init__(
        self,
        host,
        port,
        password=None,
        config=None,
        user_id=None,
        fluent_sender=None,
    ):
        RedisFluentLoggerBase.__init__(
            self, config=config, user_id=user_id, fluent_sender=fluent_sender
        )
        # Connect to a redis database
        self.db = RedisResourceInterface()
        redis_args = (host, port)
        if password is not None:
            redis_args = (*redis_args, password)
        self.db.connect(*redis_args)
        del redis_args

    @staticmethod
    def _generate_db_resource_id(user_id, resource_id, iteration=None):
        """Generate DB resource id

        Args:
            user_id (str): The user id
            resource_id (str): The resource id
            iteration (int): The iteration of the job

        Returns:
            str: The generated DB resource id

        """
        if iteration == 1:
            iteration = None
        if iteration is None:
            return "%s/%s" % (user_id, resource_id)
        else:
            return "%s/%s/%d" % (user_id, resource_id, iteration)

    @staticmethod
    def _get_iteration_from_db_resource_id(db_resource_id):
        resoucre_id_split = db_resource_id.split("/")
        if len(resoucre_id_split) == 3:
            return int(resoucre_id_split[2])
        else:
            return 1

    def commit(
        self, user_id, resource_id, iteration, document, expiration=8640000
    ):
        """Commit a resource entry to the database, create a new one if it
        does not exists, update existing resource entries

        Args:
            user_id (str): The user id
            resource_id (str): The resource id
            iteration (int): The iteration of the job
            document (str): The pickled document to store in the database
            expiration (int): Number of seconds of expiration time, default
                              8640000s hence 100 days

        Returns:
            bool:
            True for success, False otherwise

        """

        db_resource_id = self._generate_db_resource_id(
            user_id, resource_id, iteration
        )
        redis_return = bool(self.db.set(db_resource_id, document, expiration))
        http_code, data = pickle.loads(document)
        data["logger"] = "resources_logger"
        self.send_to_logger("RESOURCE_LOG", data)
        return redis_return

    def commit_termination(
        self, user_id, resource_id, iteration=None, expiration=3600
    ):
        """Commit a resource entry to the database that requires the
        termination of the resource, create a new one if it does not exists,
        update existing resource entries

        Args:
            user_id (str): The user id
            resource_id (str): The resource id
            iteration (int): The iteration of the job
            expiration (int): Number of seconds of expiration time, default
                              3600 hence 1 hour

        Returns:
            bool:
            True for success, False otherwise

        """
        db_resource_id = self._generate_db_resource_id(
            user_id, resource_id, iteration
        )
        return bool(self.db.set_termination(db_resource_id, expiration))

    def get(self, user_id, resource_id, iteration=None):
        """Get resource entry

        Args:
            user_id (str): The user id
            resource_id (str): The resource id
            iteration (int): The iteration of the job

        Returns:
            str:
            The resource document or None

        """
        db_resource_id = self._generate_db_resource_id(
            user_id, resource_id, iteration
        )
        return self.db.get(db_resource_id)

    def get_latest_iteration(self, user_id, resource_id=None):
        """Get resource entry with latest iteration

        Args:
            user_id (str): The user id
            resource_id (str): The resource id

        Returns:
            int:
            The latest iteration of the resource
            str:
            The resource document or None

        """
        db_resource_id_pattern = "%s*" % self._generate_db_resource_id(
            user_id, resource_id, None
        )
        db_keys = self.db.get_keys_from_pattern(db_resource_id_pattern)
        if len(db_keys) == 1:
            db_resource_id = db_keys[0]
        else:
            db_resource_id = max(db_keys)
        iteration = self._get_iteration_from_db_resource_id(db_resource_id)
        if iteration == 1:
            iteration = None
        return iteration, self.db.get(db_resource_id)

    def get_all_iteration(self, user_id, resource_id):
        """Get resource entry of all iterations

        Args:
            user_id (str): The user id
            resource_id (str): The resource id

        Returns:
            str:
            The resource document or None

        """
        db_resource_id = self._generate_db_resource_id(
            user_id, resource_id, None
        )
        db_resource_id_pattern = "%s*" % db_resource_id
        db_keys = self.db.get_keys_from_pattern(db_resource_id_pattern)
        db_keys.sort()
        resp_dict = dict()
        for db_key in db_keys:
            iteration = self._get_iteration_from_db_resource_id(db_key)
            if iteration != 1:
                db_resource_id_iter = self._generate_db_resource_id(
                    user_id, resource_id, iteration
                )
            else:
                db_resource_id_iter = db_resource_id
            resp_dict[str(iteration)] = pickle.loads(
                self.db.get(db_resource_id_iter)
            )[1]
        return pickle.dumps([200, resp_dict])

    def get_user_resources(self, user_id):
        """Get a user specific list of resource entries

        Args:
            user_id (str): The user id

        Returns:
            list:
            A list of resource document

        """
        resource_list_pickled = self.db.get_list(user_id + "*")
        resource_list = []

        if resource_list_pickled:
            for entry in resource_list_pickled:
                http_code, data = pickle.loads(entry)
                resource_list.append(data)

        return resource_list

    def get_all_resources(self):
        """Get all resource entries

        Returns:
            list:
            A list resource document

        """

        resource_list_pickled = self.db.get_list("*")
        resource_list = []

        if resource_list_pickled:
            for entry in resource_list_pickled:
                http_code, data = pickle.loads(entry)
                resource_list.append(data)

        return resource_list

    def get_termination(self, user_id, resource_id, iteration=None):
        """Get resource entry that requires the termination of the resource

        Args:
            user_id (str): The user id
            resource_id (str): The resource id
            iteration (int): The iteration of the job

        Returns:
            bool:
            True is resource should be terminated or False if otherwise

        """
        db_resource_id = self._generate_db_resource_id(
            user_id, resource_id, iteration
        )
        return self.db.get_termination(db_resource_id)

    def delete(self, user_id, resource_id, iteration=None):
        """Delete resource entry

        Args:
            user_id (str): The user id
            resource_id (str): The resource id
            iteration (int): The iteration of the job

        Returns:
            bool:
            True for success, False otherwise

        """
        db_resource_id = self._generate_db_resource_id(
            user_id, resource_id, iteration
        )
        return bool(self.db.delete(db_resource_id))

    def delete_termination(self, user_id, resource_id, iteration=None):
        """Delete resource termination entry

        Args:
            user_id (str): The user id
            resource_id (str): The resource id
            iteration (int): The iteration of the job

        Returns:
            bool:
            True for success, False otherwise

        """
        db_resource_id = self._generate_db_resource_id(
            user_id, resource_id, iteration
        )
        return bool(self.db.delete_termination(db_resource_id))



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Google Big Query interface test case
"""
from actinia_core.core.common.google_satellite_bigquery_interface import (
    GoogleSatelliteBigQueryInterface,
)
from actinia_core.core.common.config import global_config
import os
import unittest
from pprint import pprint

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Soeren Gebbert"
__email__ = "soerengebbert@googlemail.com"


class GoogleBigQueryInterfaceTestCase(unittest.TestCase):
    """This class tests the BigQuery google interface to collect download
    urls of landsat and sentinel data
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_query_for_landsat_scene_time(self):
        gsqi = GoogleSatelliteBigQueryInterface(global_config)

        start = "2017-01-01T10:45:00"
        end = "2017-01-01T10:55:00"

        result = gsqi.query_landsat_archive(start_time=start, end_time=end)
        pprint(result)
        self.assertTrue(len(result) >= 1)

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_query_for_landsat_scene_time_coords(self):
        gsqi = GoogleSatelliteBigQueryInterface(global_config)

        start = "2016-11-01T00:00:00"
        end = "2016-11-03T00:00:00"

        result = gsqi.query_landsat_archive(
            start_time=start, end_time=end, lon=-45.0, lat=-7.0
        )
        pprint(result)
        self.assertTrue(len(result) >= 1)

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_query_for_sentinel2_scene_time(self):
        gsqi = GoogleSatelliteBigQueryInterface(global_config)

        start = "2017-01-01T10:05:00"
        end = "2017-01-01T11:00:00"

        result = gsqi.query_sentinel2_archive(start_time=start, end_time=end)
        pprint(result)
        self.assertEqual(len(result), 154)

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_query_for_sentinel2_scene_time_coords(self):
        gsqi = GoogleSatelliteBigQueryInterface(global_config)

        start = "2016-11-01T00:00:00"
        end = "2016-11-30T00:00:00"

        result = gsqi.query_sentinel2_archive(
            start_time=start, end_time=end, lon=-45.0, lat=-7.0
        )
        pprint(result)
        self.assertTrue(len(result) >= 1)

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_query_for_sentinel_scene_sinlge(self):
        gsqi = GoogleSatelliteBigQueryInterface(global_config)
        # A very small scene
        result = gsqi.get_sentinel_urls(
            [
                "S2A_MSIL1C_20170212T104141_N0204_R008_T31TGJ_20170212T104138",
            ],
            ["B12", "B08"],
        )
        pprint(result)
        self.assertTrue(len(result) == 1)

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_query_for_sentinel_scene_four(self):
        gsqi = GoogleSatelliteBigQueryInterface(global_config)
        result = gsqi.get_sentinel_urls(
            [
                "S2A_MSIL1C_20170202T090201_N0204_R007_T36TVT_20170202T090155",
                "S2A_MSIL1C_20170221T093031_N0204_R136_T34UGV_20170221T093310",
                "S2A_MSIL1C_20170211T060951_N0204_R134_T44WPT_20170211T060950",
                "S2A_MSIL1C_20170218T143751_N0204_R096_T20PRT_20170218T143931",
            ],
            ["B04", "B08"],
        )
        pprint(result)
        self.assertTrue(len(result) == 4)

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_query_for_sentinel_scene_two_missing(self):
        gsqi = GoogleSatelliteBigQueryInterface(global_config)
        result = gsqi.get_sentinel_urls(
            [
                "S2A_MSIL1C_20170301T225331_N0204_R115_T56DNG_20170301T225347",
                "S2A_MSIL1C_20170221T093031_N0204_R136_T34UGV_20170221T093310",
                "S2A_MSIL1C_NOPE",
                "S2A_MSIL1C_POPE",
            ],
            ["B04", "B08"],
        )
        pprint(result)
        self.assertTrue(len(result) == 2)

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_query_for_sentinel_scene_empty_query(self):
        gsqi = GoogleSatelliteBigQueryInterface(global_config)
        result = gsqi.get_sentinel_urls(
            [
                "S2A_MSIL1C_20170208T092131_N0204_R093_T35TLF_NOPENOPENOPE",
            ],
            ["B04", "B08"],
        )
        pprint(result)
        self.assertTrue(not result)

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_query_for_landsat_scene(self):
        gsqi = GoogleSatelliteBigQueryInterface(global_config)
        result = gsqi.get_landsat_urls(
            [
                "LC80440342016259LGN00",
                "LC80440342013106LGN01",
                "LC80440342013154LGN00",
            ],
            ["B1", "MTL"],
        )
        pprint(result)
        self.assertTrue(len(result) == 3)

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_query_for_landsat_scene_one_missing(self):
        gsqi = GoogleSatelliteBigQueryInterface(global_config)
        result = gsqi.get_landsat_urls(
            [
                "LC80440342016259LGN00",
                "LC80440342013106LGN01",
                "LC80440342013154LGN00_WRONG",
            ],
            ["B1", "MTL"],
        )
        pprint(result)
        self.assertTrue(len(result) == 2)

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_query_for_landsat_scene_empty_query(self):
        gsqi = GoogleSatelliteBigQueryInterface(global_config)
        result = gsqi.get_landsat_urls(
            [
                "LE72_NOPENOPENOPE",
            ],
            ["B1", "MTL"],
        )
        pprint(result)
        self.assertTrue(not result)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Entpoint configuration
"""

import csv
from flask import jsonify
from flask import make_response
from functools import wraps
import os
import sys

from actinia_core.core.common.config import global_config
from actinia_core.models.response_models import SimpleResponseModel


__license__ = "GPLv3"
__author__ = "Anika Weinmann"
__copyright__ = "Copyright 2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"


if global_config.ENDPOINTS_CONFIG is not None and os.path.isfile(
    global_config.ENDPOINTS_CONFIG
):
    with open(global_config.ENDPOINTS_CONFIG, mode="r") as inp:
        reader = csv.reader(inp, delimiter=";")
        endpoints_dict = {
            row[0]: [method.upper() for method in row[1].split(",")]
            for row in reader
            if len(row) == 2
        }
else:
    endpoints_dict = None


def check_endpoint(method, api_doc=None, endpoint_class=None):
    if endpoint_class is None:
        endpoint_class = sys._getframe().f_back.f_code.co_name
    method_upper = method.upper()
    if endpoints_dict is None or (
        endpoint_class in endpoints_dict
        and method_upper in endpoints_dict[endpoint_class]
    ):
        return api_doc if api_doc is not None else True
    else:
        return False


def endpoint_decorator():
    """Check whether an endpoint allowlist should be used to
    allow only the provided list of endpoints and hide all others.
    """

    def decorator(func):
        endpoint_class, method = func.__qualname__.split(".")

        @wraps(func)
        def wrapper(*args, **kwargs):
            if check_endpoint(method, endpoint_class=endpoint_class):
                result = func(*args, **kwargs)
                return result
            else:
                return make_response(
                    jsonify(
                        SimpleResponseModel(
                            status="error",
                            message="Not Found. The requested URL is not "
                            "configured on the server.",
                        )
                    ),
                    404,
                )

        return wrapper

    return decorator


# def check_queue_type_overwrite():
#     """Always use local queue for synchronous requests to respond in time.
#     Decorator is used here so it is more easy to change later.
#     """

#     def decorator(func):

#         @wraps(func)
#         def wrapper(*args, **kwargs):
#             result = func(*args, **kwargs, queue_type_overwrite='local')
#             return result

#         return wrapper

#     return decorator



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Landsat processing commands
"""
import os
from .process_object import Process
from actinia_core.core.geodata_download_importer import (
    GeoDataDownloadImportSupport,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"

SUPPORTED_MIMETYPES = [
    "application/zip",
    "application/tiff",
    "application/gml",
]

SCENE_SUFFIXES = {
    "LT04": [
        "_B1.TIF",
        "_B2.TIF",
        "_B3.TIF",
        "_B4.TIF",
        "_B5.TIF",
        "_B6.TIF",
        "_B7.TIF",
        "_MTL.txt",
    ],
    "LT05": [
        "_B1.TIF",
        "_B2.TIF",
        "_B3.TIF",
        "_B4.TIF",
        "_B5.TIF",
        "_B6.TIF",
        "_B7.TIF",
        "_MTL.txt",
    ],
    "LE07": [
        "_B1.TIF",
        "_B2.TIF",
        "_B3.TIF",
        "_B4.TIF",
        "_B5.TIF",
        "_B6_VCID_2.TIF",
        "_B6_VCID_1.TIF",
        "_B7.TIF",
        "_B8.TIF",
        "_MTL.txt",
    ],
    "LC08": [
        "_B1.TIF",
        "_B2.TIF",
        "_B3.TIF",
        "_B4.TIF",
        "_B5.TIF",
        "_B6.TIF",
        "_B7.TIF",
        "_B8.TIF",
        "_B9.TIF",
        "_B10.TIF",
        "_B11.TIF",
        "_MTL.txt",
    ],
}

RASTER_SUFFIXES = {
    "LT04": [".1", ".2", ".3", ".4", ".5", ".6", ".7"],
    "LT05": [".1", ".2", ".3", ".4", ".5", ".6", ".7"],
    "LE07": [".1", ".2", ".3", ".4", ".5", ".61", ".62", ".7", ".8"],
    "LC08": [
        ".1",
        ".2",
        ".3",
        ".4",
        ".5",
        ".6",
        ".7",
        ".8",
        ".9",
        ".10",
        ".11",
    ],
}


SCENE_BANDS = {
    "LT04": ["B1", "B2", "B3", "B4", "B5", "B6", "B7", "MTL"],
    "LT05": ["B1", "B2", "B3", "B4", "B5", "B6", "B7", "MTL"],
    "LE07": [
        "B1",
        "B2",
        "B3",
        "B4",
        "B5",
        "B6_VCID_2",
        "B6_VCID_1",
        "B7",
        "B8",
        "MTL",
    ],
    "LC08": [
        "B1",
        "B2",
        "B3",
        "B4",
        "B5",
        "B6",
        "B7",
        "B8",
        "B9",
        "B10",
        "B11",
        "MTL",
    ],
}


def extract_sensor_id_from_scene_id(scene_id):
    """Extract the sensor id from a Landsat scene id

    Args:
        scene_id (str): The landsat scene id

    Returns:
        (str)
        The sencor id

    """
    return scene_id.split("_")[0]


def scene_id_to_google_url(scene_id, suffix):
    """Convert a landsat scene id into the public google download URL for the
    required file

    Args:
        scene_id (str): The Landsat scene id
        suffix (str): The suffix of the file to create the url for,  i.e.:
                      "_B1.TIF" or "_MTL.txt"
    Returns:
        (str)
        The URL to the scene file
    """
    # new URL example
    # https://storage.googleapis.com/gcp-public-data-landsat/LC08/01/001/004/
    # LC08_L1GT_001004_20130910_20170502_01_T2/
    # LC08_L1GT_001004_20130910_20170502_01_T2_B1.TIF

    # Create the download URL components from the Landsat scene id
    landsat_sensor_id = extract_sensor_id_from_scene_id(scene_id)
    path = scene_id.split("_")[2][:3]
    row = scene_id.split("_")[2][3:]

    url = (
        "https://storage.googleapis.com/gcp-public-data-landsat/"
        f"{landsat_sensor_id}/01/{path}/{row}/{scene_id}/{scene_id}{suffix}"
    )
    return url


def datetime_to_grass_datetime_string(dt):
    """Convert a python datetime object into a GRASS datetime string"""
    # GRASS datetime month names
    month_names = [
        "",
        "jan",
        "feb",
        "mar",
        "apr",
        "may",
        "jun",
        "jul",
        "aug",
        "sep",
        "oct",
        "nov",
        "dec",
    ]

    # Check for time zone info in the datetime object
    if dt.tzinfo is not None:
        tz = dt.tzinfo.utcoffset(0)
        if tz.seconds > 86400 / 2:
            tz = (tz.seconds - 86400) / 60
        else:
            tz = tz.seconds / 60

        string = "%.2i %s %.2i %.2i:%.2i:%.2i %+.4i" % (
            dt.day,
            month_names[dt.month],
            dt.year,
            dt.hour,
            dt.minute,
            dt.second,
            tz,
        )
    else:
        string = "%.2i %s %.4i %.2i:%.2i:%.2i" % (
            dt.day,
            month_names[dt.month],
            dt.year,
            dt.hour,
            dt.minute,
            dt.second,
        )

    return string


class LandsatProcessing(GeoDataDownloadImportSupport):
    """"""

    def __init__(
        self,
        config,
        scene_id,
        temp_file_path,
        download_cache,
        send_resource_update,
        message_logger,
    ):
        """A collection of functions to generate Landsat4-8 scene related
        import and processing commands. Each function returns a process chain
        that can be executed by the async processing classes.

        Args:
            config: The Actinia Core configuration object
            scene_id (str): The scene id for which all bands should be
                            downloaded
            temp_file_path: The path to the temporary directory to store
                            temporary files. It is assumed that this path is
                            available when the generated commands are executed.
            download_cache (str): The path to the download cache
            send_resource_update: The function to call for resource updates
            message_logger: The message logger to be used

        """

        GeoDataDownloadImportSupport.__init__(
            self,
            config,
            temp_file_path,
            download_cache,
            send_resource_update,
            message_logger,
            None,
        )

        self.scene_id = scene_id
        self.landsat_sensor_id = None
        self.url_list = []
        self.raster_names = []
        self.band_raster_names = {}
        self.ndvi_name = None

    def _setup(self):
        """
        Setup the download of the required Landsat scene from the Google Cloud
        Storage.

        Check the download cache if the file already exists, to avoid redundant
        downloads. The downloaded files will be stored in a temporary
        directory. After the download of all files completes, the downloaded
        files will be moved to the download cache. This avoids broken files in
        case a download was interrupted or stopped by termination.

        This method uses wget to gather the landsat scenes from the Google
        Cloud Storage landsat archive using public https address.
        """

        GeoDataDownloadImportSupport._setup(self)
        self.landsat_sensor_id = extract_sensor_id_from_scene_id(self.scene_id)

        count = 0
        # Create file names, urls and check the download cache
        for suffix in SCENE_SUFFIXES[self.landsat_sensor_id]:
            file_name = "%s%s" % (self.scene_id, suffix)
            # This is the file path in the download cache directory
            file_path = os.path.join(self.user_download_cache_path, file_name)
            self.file_list.append(file_path)
            # This is the download path
            temp_file_path = os.path.join(self.temp_file_path, file_name)
            # Create the download URL
            url = scene_id_to_google_url(self.scene_id, suffix)

            self.url_list.append(url)
            self.copy_file_list.append((temp_file_path, file_path))
            count += 1

    def get_import_process_list(self):
        count = 0
        import_commands = []

        for file_path in self.file_list:
            if "_MTL.TXT" not in file_path.upper():
                raster_name = "%s%s" % (
                    self.scene_id,
                    RASTER_SUFFIXES[self.landsat_sensor_id][count],
                )
                self.raster_names.append(raster_name)
                self.band_raster_names[
                    SCENE_BANDS[self.landsat_sensor_id][count]
                ] = raster_name
                p = self.get_raster_import_command(
                    file_path=file_path, raster_name=raster_name
                )
                import_commands.append(p)
                count += 1

        return import_commands

    def get_i_landsat_toar_process_list(self, atcor_method):
        option = "uncorrected"

        if atcor_method == "DOS4":
            option = "dos4"

        if atcor_method == "DOS1":
            option = "dos1"

        toar_commands = []

        p = Process(
            exec_type="grass",
            executable="i.landsat.toar",
            executable_params=[
                "input=%s." % self.scene_id,
                "metfile=%s_%s"
                % (
                    os.path.join(self.user_download_cache_path, self.scene_id),
                    "MTL.txt",
                ),
                "method=%s" % option,
                "output=%s_%s." % (self.scene_id, atcor_method),
                "--q",
            ],
            id=f"top_of_atmosphere_{self.scene_id}",
            skip_permission_check=True,
        )
        toar_commands.append(p)
        return toar_commands

    def get_i_vi_process_list(self, atcor_method, processing_method):
        self.ndvi_name = "%s_%s_%s" % (
            self.scene_id,
            atcor_method,
            processing_method,
        )

        ndvi_commands = []

        ivi = "i.vi"
        ivi_params = list()
        if self.landsat_sensor_id == "LC08":
            ivi_params.append(
                "red=%s_%s%s" % (self.scene_id, atcor_method, ".4")
            )
            ivi_params.append(
                "nir=%s_%s%s" % (self.scene_id, atcor_method, ".5")
            )
            ivi_params.append(
                "green=%s_%s%s" % (self.scene_id, atcor_method, ".3")
            )
            ivi_params.append(
                "blue=%s_%s%s" % (self.scene_id, atcor_method, ".2")
            )
            ivi_params.append(
                "band5=%s_%s%s" % (self.scene_id, atcor_method, ".7")
            )
            ivi_params.append(
                "band7=%s_%s%s" % (self.scene_id, atcor_method, ".8")
            )
        else:
            ivi_params.append(
                "red=%s_%s%s" % (self.scene_id, atcor_method, ".3")
            )
            ivi_params.append(
                "nir=%s_%s%s" % (self.scene_id, atcor_method, ".4")
            )
            ivi_params.append(
                "green=%s_%s%s" % (self.scene_id, atcor_method, ".2")
            )
            ivi_params.append(
                "blue=%s_%s%s" % (self.scene_id, atcor_method, ".1")
            )
            ivi_params.append(
                "band5=%s_%s%s" % (self.scene_id, atcor_method, ".5")
            )
            ivi_params.append(
                "band7=%s_%s%s" % (self.scene_id, atcor_method, ".7")
            )

        ivi_params.append("viname=%s" % processing_method.lower())
        ivi_params.append("output=%s" % self.ndvi_name)

        p = Process(
            exec_type="grass",
            executable=ivi,
            executable_params=ivi_params,
            id=f"i_vi_{processing_method.lower()}_{self.ndvi_name}",
            skip_permission_check=True,
        )
        ndvi_commands.append(p)

        p = Process(
            exec_type="grass",
            executable="r.colors",
            executable_params=["map=%s" % self.ndvi_name, "color=ndvi"],
            id=f"set_colors_{processing_method.lower()}_{self.ndvi_name}",
            skip_permission_check=True,
        )
        ndvi_commands.append(p)

        return ndvi_commands



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Async process test case
"""
import unittest
import copy
from flask.json import loads as json_loads, dumps as json_dumps
import time

try:
    from .test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"

# Module change example for r.slope.aspect with g.region adjustment
process_chain = {
    1: {
        "module": "g.region",
        "inputs": {"raster": "elevation@PERMANENT"},
        "flags": "p",
        "verbose": True,
    },
    2: {
        "module": "r.slope.aspect",
        "inputs": {
            "elevation": "elevation@PERMANENT",
            "format": "degrees",
            "min_slope": "0.0",
        },
        "outputs": {
            "aspect": {"name": "my_aspect"},
            "slope": {
                "name": "my_slope",
                "export": {"format": "GTiff", "type": "raster"},
            },
        },
        "flags": "a",
        "overwrite": False,
        "verbose": True,
    },
    3: {
        "module": "r.watershed",
        "inputs": {"elevation": "elevation@PERMANENT"},
        "outputs": {
            "accumulation": {
                "name": "my_accumulation",
                "export": {"format": "GTiff", "type": "raster"},
            }
        },
    },
    4: {
        "module": "r.info",
        "inputs": {"map": "my_aspect"},
        "flags": "gr",
        "verbose": True,
    },
    5: {"executable": "/bin/true", "parameters": []},
    6: {"executable": "/bin/true"},
    7: {"executable": "/bin/sleep", "parameters": ["4"]},
}

# Module chains with errors
process_chain_error_1 = {
    1: {
        "module": "g.region",
        "inputs": {"raster": "elevation@PERMANENT"},
        "flags": "&p",
        "verbose": True,
    }
}

process_chain_error_2 = {
    1: {
        "module": "g.region_nopo",
        "inputs": {"raster": "elevation@PERMANENT"},
        "flags": "p",
        "verbose": True,
    }
}

process_chain_error_3 = {
    1: {
        "module": "g.region",
        "inputs": {"raster": "elevion@PANENT"},
        "flags": "p",
        "verbose": True,
    }
}

process_chain_error_4 = {
    1: {
        "module": "g.region",
        "inputs": {"faster": "elevion@PANENT"},
        "flags": "p",
        "verbose": True,
    }
}

process_chain_error_5 = {1: {}}

process_chain_error_6 = {
    2: {
        "module": "r.slope.aspect",
        "inputs": {
            "elevation": "elevation@PERMANENT",
            "format": "degrees",
            "min_slope": "0.0",
        },
        "outputs": {
            "aspect": {"name": "my_aspect"},
            "slope": {
                "name": "my_slope",
                "export": {"fromat": "GTiff", "type": "raster"},
            },
        },
        "flags": "a",
        "overwrite": False,
        "verbose": True,
    },
}

process_chain_region = {
    1: {
        "module": "g.region",
        "inputs": {"raster": "elevation@PERMANENT"},
        "flags": "p",
        "verbose": True,
    },
    2: {
        "module": "g.region",
        "inputs": {"res": "0.001"},
        "flags": "p",
        "verbose": True,
    },
    3: {
        "module": "r.info",
        "inputs": {"map": "elevation@PERMANENT"},
        "flags": "gr",
        "verbose": True,
    },
}


class AsyncProcessTestCase(ActiniaResourceTestCaseBase):
    def test_async_processing(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

    def test_async_processing_termination(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain),
            content_type="application/json",
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        resp = json_loads(rv.data)

        rv_user_id = resp["user_id"]
        rv_resource_id = resp["resource_id"]

        while True:
            rv = self.server.get(
                URL_PREFIX + "/resources/%s/%s" % (rv_user_id, rv_resource_id),
                headers=self.admin_auth_header,
            )
            print("Get", rv.data.decode())
            resp = json_loads(rv.data)
            if (
                resp["status"] == "finished"
                or resp["status"] == "error"
                or resp["status"] == "terminated"
            ):
                break

            time.sleep(0.2)

            # Send the termination request, again and again :)
            rv = self.server.delete(
                URL_PREFIX + "/resources/%s/%s" % (rv_user_id, rv_resource_id),
                headers=self.admin_auth_header,
            )
            print("Delete", rv.data.decode())

        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )
        self.assertEqual(resp["status"], "terminated")

        time.sleep(1)

    def test_async_processing_large_region(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.user_auth_header,
            data=json_dumps(process_chain_region),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=400,
            status="error",
            message_check="AsyncProcessError:",
        )

    def test_async_processing_error_1(self):
        """
        Wrong character in module description
        :return:
        """

        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.user_auth_header,
            data=json_dumps(process_chain_error_1),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=400,
            status="error",
            message_check="AsyncProcessError:",
        )

    def test_async_processing_error_2(self):
        """
        Wrong module error
        :return:
        """

        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.user_auth_header,
            data=json_dumps(process_chain_error_2),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=400,
            status="error",
            message_check="AsyncProcessError:",
        )

    def test_async_processing_error_3(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.user_auth_header,
            data=json_dumps(process_chain_error_3),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=400,
            status="error",
            message_check="AsyncProcessError:",
        )

    def test_async_processing_error_4(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.user_auth_header,
            data=json_dumps(process_chain_error_4),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=400,
            status="error",
            message_check="AsyncProcessError:",
        )

    def test_async_processing_error_5(self):
        """No JSON payload error"""

        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.user_auth_header,
        )

        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_async_processing_error_6(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.user_auth_header,
            data=json_dumps(process_chain_error_5),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=400,
            status="error",
            message_check="AsyncProcessError:",
        )

    def test_async_processing_error_7(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.user_auth_header,
            data=json_dumps(process_chain_error_6),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=400,
            status="error",
            message_check="AsyncProcessError:",
        )

    def test_async_processing_error_8(self):
        """Wrong mapset in input definitions as admin"""

        pc = copy.deepcopy(process_chain)
        pc[1]["inputs"]["raster"] = "elevation@NO_Mapset"
        pc[2]["inputs"]["elevation"] = "elevation@NO_Mapset"
        pc[3]["inputs"]["elevation"] = "elevation@NO_Mapset"

        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.admin_auth_header,
            data=json_dumps(pc),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=400,
            status="error",
            message_check="AsyncProcessError:",
        )

    def test_async_processing_error_9(self):
        """
        Wrong mapset in input definitions as user, hence authorization error
        """

        pc = copy.deepcopy(process_chain_error_1)
        pc[1]["inputs"]["raster"] = "elevation@NO_Mapset"
        pc[1]["flags"] = "p"

        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async",
            headers=self.user_auth_header,
            data=json_dumps(pc),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=400,
            status="error",
            message_check="AsyncProcessError:",
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Async process export2 test case admin
"""
import os
import unittest
from flask.json import loads as json_loads, dumps as json_dumps
import requests

try:
    from .test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"

# Module change example for r.slope.aspect with g.region adjustment
process_chain_long = {
    1: {
        "module": "g.region",
        "inputs": {"raster": "elevation@PERMANENT", "res": "1000"},
        "flags": "p",
        "verbose": True,
    },
    2: {
        "module": "r.slope.aspect",
        "inputs": {
            "elevation": "elevation@PERMANENT",
            "format": "degrees",
            "min_slope": "0.0",
        },
        "outputs": {
            "aspect": {
                "name": "my_aspect",
                "export": {"format": "GTiff", "type": "raster"},
            },
            "slope": {
                "name": "my_slope",
                "export": {"format": "GTiff", "type": "raster"},
            },
        },
        "flags": "a",
        "overwrite": False,
        "verbose": True,
    },
    3: {
        "module": "r.watershed",
        "inputs": {"elevation": "elevation@PERMANENT"},
        "outputs": {
            "accumulation": {
                "name": "my_accumulation",
                "export": {"format": "GTiff", "type": "raster"},
            }
        },
    },
}


class AsyncProcessExport2TestCaseAdmin(ActiniaResourceTestCaseBase):
    def test_async_processing_export(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_long),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header
        )

        # Get the exported results
        urls = resp["urls"]["resources"]

        for url in urls:
            print(url)
            rv = self.server.get(url, headers=self.user_auth_header)
            self.assertEqual(
                rv.status_code,
                200,
                "HTML status code is wrong %i" % rv.status_code,
            )
            self.assertEqual(
                rv.mimetype, "image/tiff", "Wrong mimetype %s" % rv.mimetype
            )
            print(rv.headers)
            print(rv.iter_encoded())


class AsyncProcessExportTestCaseAdminS3(ActiniaResourceTestCaseBase):
    """Test the S3 storage support

    Deactivated due to credential exposition problem
    """

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_async_processing_export(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export_s3",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_long),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header
        )

        # Get the exported results
        urls = resp["urls"]["resources"]

        for url in urls:
            print(url)
            rv = requests.get(url)
            print(rv)
            self.assertEqual(
                rv.status_code,
                200,
                "HTML status code is wrong %i" % rv.status_code,
            )

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_termination(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export_s3",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_long),
            content_type="application/json",
        )
        resp = json_loads(rv.data)
        # Send the termination request
        self.server.delete(
            URL_PREFIX
            + "/resources/%s/%s" % (resp["user_id"], resp["resource_id"]),
            headers=self.admin_auth_header,
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="terminated",
            message_check="AsyncProcessTermination:",
        )


class AsyncProcessExportTestCaseAdminGCS(ActiniaResourceTestCaseBase):
    """Test the GCS storage support

    Deactivated due to credential exposition problem
    """

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_async_processing_export(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export_gcs",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_long),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header
        )

        # Get the exported results
        urls = resp["urls"]["resources"]

        for url in urls:
            print(url)
            rv = requests.get(url)
            print(rv)
            self.assertEqual(
                rv.status_code,
                200,
                "HTML status code is wrong %i" % rv.status_code,
            )

    @unittest.skipIf(
        "GOOGLE_APPLICATION_CREDENTIALS" not in os.environ
        and "GOOGLE_CLOUD_PROJECT" not in os.environ,
        "Test is skipped because 'GOOGLE_APPLICATION_CREDENTIALS' and "
        "'GOOGLE_CLOUD_PROJECT' not set",
    )
    def test_termination(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export_gcs",
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_long),
            content_type="application/json",
        )
        resp = json_loads(rv.data)
        # Send the termination request
        self.server.delete(
            URL_PREFIX
            + "/resources/%s/%s" % (resp["user_id"], resp["resource_id"]),
            headers=self.admin_auth_header,
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="terminated",
            message_check="AsyncProcessTermination:",
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2021 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Process chain
"""
import os
import requests

from actinia_core.core.stac_importer_interface import STACImporter as STAC
from .process_object import Process
from .exceptions import AsyncProcessError
from actinia_core.core.geodata_download_importer import (
    GeoDataDownloadImportSupport,
)
from .config import global_config
from .sentinel_processing_library import Sentinel2Processing
from .landsat_processing_library import LandsatProcessing
from .google_satellite_bigquery_interface import (
    GoogleSatelliteBigQueryInterface,
)
from requests.auth import HTTPBasicAuth
from actinia_core.models.process_chain import (
    ProcessChainModel,
    InputParameter,
    OutputParameter,
    GrassModule,
    StdoutParser,
    Executable,
    SUPPORTED_EXPORT_FORMATS,
)

__license__ = "GPLv3"
__author__ = (
    "Sören Gebbert, Carmen Tawalika, Guido Riembauer, Julia Haas,"
    " Anika Weinmann"
)
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class ProcessChainConverter(object):
    """
    Convert the process chain description into a process list that can be
    executed
    """

    def __init__(
        self,
        config=None,
        temp_file_path=None,
        process_dict=None,
        temporary_pc_files=None,
        required_mapsets=None,
        resource_export_list=None,
        message_logger=None,
        output_parser_list=None,
        send_resource_update=None,
    ):
        """Constructor to convert the process chain into a process list

        Args:
            config: The actinia configuration object
            temp_file_path: The path to the temporary directory to store
                            temporary files. It is assumed that this path is
                            available when the generated commands are executed.
            message_logger: The message logger to be used
            process_dict (dict): The dictionary that describes the process
                                 chain
            temporary_pc_files (dict): A dictionary of temporary process chain
                                       files that are generated by this class
            required_mapsets (list): A list that will be filled with mapsets
                                     names that must be linked for processing
            resource_export_list (list): A list that will be filled with export
                                         definitions found in output
                                         descriptions for geodata export
            output_parser_list (list): A list that will be filled with output
                                       parser definitions found in module
                                       descriptions. The stdout definition will
                                       be stored in a dict that has the process
                                       id a key {process_id:StdoutParser}
            send_resource_update: The function to call for resource updates

        Returns:

        """
        if config is None:
            self.config = global_config
        else:
            self.config = config
        if temp_file_path is None:
            self.temp_file_path = "/tmp"
        else:
            self.temp_file_path = temp_file_path
        self.temp_file_count = 0

        if process_dict is None:
            self.process_dict = {}
        else:
            self.process_dict = process_dict
        if temporary_pc_files is None:
            self.temporary_pc_files = {}
        else:
            self.temporary_pc_files = temporary_pc_files
        if required_mapsets is None:
            self.required_mapsets = []
        else:
            self.required_mapsets = required_mapsets
        if resource_export_list is None:
            self.resource_export_list = []
        else:
            self.resource_export_list = resource_export_list
        if output_parser_list is None:
            self.output_parser_list = []
        else:
            self.output_parser_list = output_parser_list

        self.send_resource_update = send_resource_update
        self.message_logger = message_logger
        self.import_descr_list = []
        self.webhook_finished = None
        self.webhook_update = None
        self.webhook_auth = None

    def process_chain_to_process_list(self, process_chain):
        if not process_chain:
            raise AsyncProcessError("Process chain is empty")

        if "list" in process_chain and "version" in process_chain:
            return self._process_chain_to_process_list(process_chain)
        else:
            return self._process_chain_to_process_list_legacy(process_chain)

    def _process_chain_to_process_list(self, process_chain):
        """
        Transform the module chain description into an ordered list of process
        runs

        All input map layer from a specific mapset
        MUST be specified with the mapset they belong to: map@mapset.
        Maps from an ephemeral mapset should not contain a mapset name,
        because the mapset names are temporary and created with random names.

        Args:
            process_chain (dict): The process chain

        Return:
             list
             A list of ordered grass processes

        """
        process_list = []

        if "version" not in process_chain:
            raise AsyncProcessError(
                "Version information is missing "
                "in the process chain definition"
            )

        if "list" not in process_chain:
            raise AsyncProcessError(
                "List of processes to be executed is missing "
                "in the process chain definition"
            )

        # Check for the webhooks
        if "webhooks" in process_chain:
            if "finished" in process_chain["webhooks"]:
                self.webhook_finished = process_chain["webhooks"]["finished"]
                self._check_if_webhook_exists(
                    self.webhook_finished, process_chain, "finished"
                )
            else:
                raise AsyncProcessError(
                    "The finished URL is missing in the webhooks definition."
                )

            if "update" in process_chain["webhooks"]:
                self.webhook_update = process_chain["webhooks"]["update"]
                self._check_if_webhook_exists(
                    self.webhook_update, process_chain, "update"
                )

        for process_descr in process_chain["list"]:
            if "module" in process_descr:
                module = self._create_module_process(process_descr)
                if module:
                    process_list.append(module)
            elif "exe" in process_descr:
                exe = self._create_exec_process(process_descr)
                if exe:
                    process_list.append(exe)
            elif "evaluate" in process_descr:
                process_list.append(("python", process_descr["evaluate"]))
            else:
                raise AsyncProcessError(
                    "Unknown process description "
                    "in the process chain definition"
                )
        downimp_list = self._create_download_process_list()
        downimp_list.extend(process_list)

        return downimp_list

    def _get_landsat_import_download_commands(self, entry):
        """Helper method to get the landsat import and download commands.

        Args:
            entry (dict): Entry of the import description list

        Returns:
            landsat_commands: The landsat download and import commands
        """
        # Check for band information
        if "landsat_atcor" not in entry["import_descr"]:
            raise AsyncProcessError(
                "Atmospheric detection specification is required for Landsat "
                "scene import"
            )

        atcor = entry["import_descr"]["landsat_atcor"].lower()

        if atcor not in [
            "uncorrected",
            "dos1",
            "dos2",
            "dos2b",
            "dos3",
            "dos4",
        ]:
            raise AsyncProcessError(
                "Atmospheric detection specification is wrong for "
                "Landsat scene import."
            )

        scene = entry["import_descr"]["source"]

        lp = LandsatProcessing(
            config=self.config,
            download_cache=self.temp_file_path,
            temp_file_path=self.temp_file_path,
            message_logger=self.message_logger,
            send_resource_update=self.send_resource_update,
            scene_id=scene,
        )

        download_commands, import_file_info = lp.get_download_process_list()
        import_commands = lp.get_import_process_list()
        atcor_commands = lp.get_i_landsat_toar_process_list(atcor)
        landsat_commands = download_commands
        landsat_commands.extend(import_commands)
        landsat_commands.extend(atcor_commands)

        return landsat_commands

    def _get_sentinel_import_command(self, entry):
        """Helper method to get the sentinel import command.

        Args:
            entry (dict): Entry of the import description list

        Returns:
            sentinel_commands: The sentinel import commands
        """
        # Check for band information
        if "sentinel_band" not in entry["import_descr"]:
            raise AsyncProcessError(
                "Band specification is required for Sentinel2 scene import"
            )

        scene = entry["import_descr"]["source"]
        band = entry["import_descr"]["sentinel_band"]

        gqi = GoogleSatelliteBigQueryInterface(config=self.config)
        query_result = gqi.get_sentinel_urls(
            product_ids=[
                scene,
            ],
            bands=[
                band,
            ],
        )
        sp = Sentinel2Processing(
            config=self.config,
            bands=[
                band,
            ],
            download_cache=self.temp_file_path,
            temp_file_path=self.temp_file_path,
            message_logger=self.message_logger,
            send_resource_update=self.send_resource_update,
            product_id=scene,
            query_result=query_result,
        )

        (
            download_commands,
            import_file_info,
        ) = sp.get_sentinel2_download_process_list()
        sentinel_commands = download_commands
        import_commands = sp.get_sentinel2_import_process_list()
        sentinel_commands.extend(import_commands)

        input_file, map_name = import_file_info[band]
        p = Process(
            exec_type="grass",
            executable="g.rename",
            id=f"rename_{entry['value']}",
            executable_params=[
                "raster=%s,%s" % (map_name, entry["value"]),
            ],
        )
        sentinel_commands.append(p)

        return sentinel_commands

    def _collect_sentinel_scenes_bands(self, entries):
        """Helper Method to collect all individual scenes and bands from
        different importer modules used throughout the process chain
        """
        scenes_bands = []
        # sort by source (scene ID) and bands
        scene_ids = []
        for entry in entries:
            scene_id = entry["import_descr"]["source"]
            band = entry["import_descr"]["sentinel_band"]
            output = entry["value"]
            if scene_id not in scene_ids:
                scene_ids.append(scene_id)
                scene = {
                    "scene_id": scene_id,
                    "bands": [band],
                    "outputs": [output],
                }
                scenes_bands.append(scene)
            else:
                scindex = scene_ids.index(scene_id)
                scenes_bands[scindex]["bands"].append(band)
                scenes_bands[scindex]["outputs"].append(output)
        return scenes_bands

    def _get_sentinel_import_commands(self, entries):
        """Method to get the Sentinel download and import commands using GCS
        without login
        """
        sentinel_commands = []
        scenes_bands = self._collect_sentinel_scenes_bands(entries)
        for scene in scenes_bands:
            scene_id = scene["scene_id"]
            download_cache = f"download_cache_{scene_id}"
            self.temporary_pc_files[
                download_cache
            ] = self.generate_temp_file_path()
            sp = Sentinel2Processing(
                bands=scene["bands"],
                download_cache=self.temporary_pc_files[download_cache],
                message_logger=self.message_logger,
                send_resource_update=self.send_resource_update,
                product_id=scene_id,
            )

            (
                download_commands,
                import_file_info,
            ) = sp.get_sentinel2_download_process_list_without_query()
            scene_commands = download_commands
            import_commands = (
                sp.get_sentinel2_import_process_list_without_query()
            )
            scene_commands.extend(import_commands)
            for idx, band in enumerate(scene["bands"]):
                output = scene["outputs"][idx]
                p = Process(
                    exec_type="grass",
                    executable="g.rename",
                    id=f"rename_{scene_id}_{band}",
                    executable_params=[
                        f"raster={import_file_info[band][1]},{output}"
                    ],
                )
                scene_commands.append(p)
            sentinel_commands.extend(scene_commands)
        return sentinel_commands

    def _get_postgis_import_command(self, entry):
        """Helper method to get the import command for postgis.

        Args:
            entry (dict): Entry of the import description list

        Returns:
            import_command: The import command for postgis
        """
        dbstring = "%s" % entry["import_descr"]["source"]
        vector_name = entry["value"]
        layer = None
        if "vector_layer" in entry["import_descr"]:
            layer = entry["import_descr"]["vector_layer"]
        import_command = (
            GeoDataDownloadImportSupport.get_vector_import_command(
                input_source=dbstring,
                vector_name=vector_name,
                layer_name=layer,
            )
        )
        return import_command

    def _get_raster_vector_file_download_import_command(self, entry):
        """
        Helper method to get the download and import commands for raster/vector
        and files.

        Args:
            entry (dict): Entry of the import description list

        Returns:
            rvf_downimport_commands (list): The raster/vector/file download and
                                            import commands
        """
        rvf_downimport_commands = list()
        url = entry["import_descr"]["source"]

        if not url.startswith("/vsicurl/"):
            gdis = GeoDataDownloadImportSupport(
                config=self.config,
                temp_file_path=self.temp_file_path,
                download_cache=self.temp_file_path,
                message_logger=self.message_logger,
                send_resource_update=self.send_resource_update,
                url_list=[
                    url,
                ],
            )
            (
                download_commands,
                import_file_info,
            ) = gdis.get_download_process_list()
            rvf_downimport_commands.extend(download_commands)
            input_source = import_file_info[0][2]
        else:
            input_source = url

        map_name = entry["value"]
        layer = None

        if "vector_layer" in entry["import_descr"]:
            layer = entry["import_descr"]["vector_layer"]
        if entry["import_descr"]["type"] == "raster":
            kwargs = {"file_path": input_source, "raster_name": entry["value"]}
            resamp_opt = [
                "nearest",
                "bilinear",
                "bicubic, lanczos",
                "bilinear_f",
                "bicubic_f",
                "lanzcos_f",
            ]
            resol_opt = ["estimated", "value", "region"]

            if "resample" in entry["import_descr"]:
                if entry["import_descr"]["resample"] in resamp_opt:
                    kwargs["resample"] = entry["import_descr"]["resample"]
                else:
                    raise AsyncProcessError(
                        "Error while running executable <r.import>"
                        " Please check if parameter"
                        " <resample> is set correctly."
                    )

            if "resolution" in entry["import_descr"]:
                if entry["import_descr"]["resolution"] in resol_opt:
                    kwargs["resolution"] = entry["import_descr"]["resolution"]
                else:
                    raise AsyncProcessError(
                        "Error while running executable <r.import>."
                        " Please check if parameter"
                        " <resolution> is set correctly."
                    )
                if (
                    kwargs["resolution"] == "value"
                    and "resolution_value" not in entry["import_descr"]
                ):
                    raise AsyncProcessError(
                        "Error while running executable <r.import>."
                        " Please check if parameter"
                        " <resolution_value> is set."
                    )

            if "resolution_value" in entry["import_descr"]:
                try:
                    float(entry["import_descr"]["resolution_value"])
                    kwargs["resolution_value"] = entry["import_descr"][
                        "resolution_value"
                    ]
                except ValueError:
                    raise AsyncProcessError(
                        "Error while running executable <r.import>. Value for "
                        "parameter <resolution_value> is not a float."
                    )
                if "resolution" not in entry["import_descr"]:
                    raise AsyncProcessError(
                        "Error while running executable <r.import>. Please "
                        "check if parameter <resolution> is set."
                    )
                if (
                    "resolution" in entry["import_descr"]
                    and entry["import_descr"]["resolution"] != "value"
                ):
                    raise AsyncProcessError(
                        "Error while running executable <r.import>. Please "
                        "check if parameter <resolution> is set to <value>."
                    )

            import_command = (
                GeoDataDownloadImportSupport.get_raster_import_command(
                    **kwargs
                )
            )
            rvf_downimport_commands.append(import_command)
        if entry["import_descr"]["type"] == "vector":
            import_command = (
                GeoDataDownloadImportSupport.get_vector_import_command(
                    input_source=input_source,
                    vector_name=map_name,
                    layer_name=layer,
                )
            )
            rvf_downimport_commands.append(import_command)
        if entry["import_descr"]["type"] == "file":
            # Search for file identifiers
            if "$file" in map_name and "::" in map_name:
                file_id = map_name.split("::")[1]
                # Use the temporary file name as copy target
                if file_id in self.temporary_pc_files:
                    rename_commands = (
                        GeoDataDownloadImportSupport.get_file_rename_command(
                            import_file_info[0][2],
                            self.temporary_pc_files[file_id],
                        )
                    )
                    rvf_downimport_commands.append(rename_commands)
                else:
                    raise AsyncProcessError(
                        "A file id is required for a download file "
                        "to use it in the process chain."
                    )
            else:
                raise AsyncProcessError(
                    "A file id is required for a download file "
                    "to use it in the process chain."
                )
        return rvf_downimport_commands

    def generate_temp_file_path(self):
        """
        Generate the path of a new unique temporary file that will be removed
        when the processe finishes. The _setup() method must be called for
        correct file generate.

        Returns: The file path to a unique temporary file
        """

        if not self.temp_file_path:
            raise AsyncProcessError(
                "Unable to create temporary file, "
                "temp_file_path is not set."
            )

        self.temp_file_count += 1

        return os.path.join(
            self.temp_file_path, "temp_file_%i" % self.temp_file_count
        )

    def _create_module_process(self, module_descr):
        """Analyse a grass process description dict and create a process
        that is used to execute a GRASS GIS binary.

        - Identify the required mapsets from the input definition and store
          them in a list.
        - Identify input and output options
        - Add export options to the export list

        Args:
            module_descr (dict): The module description

        Returns: Process:
            An object of type Process that contains the module name,
            the parameter list and stdout/stdin definitions

        """
        if self.message_logger:
            self.message_logger.info(str(module_descr))
        params = []

        if "id" not in module_descr:
            raise AsyncProcessError(
                "The <id> is missing from the process description."
            )

        id = module_descr["id"]

        stdin_func = None
        if "stdin" in module_descr:
            stdin_func = self._create_stdin_process(module_descr, id)

        # Check for stdout parser that can be of type table, list or key/value
        # pairs and store the definition in a list
        if "stdout" in module_descr:
            self._check_and_get_stdout_command(module_descr, id)

        if "module" not in module_descr:
            raise AsyncProcessError(
                "Missing module name in module description of id %s" % str(id)
            )

        module_name = module_descr["module"]

        if "inputs" in module_descr:
            self._add_grass_module_input_parameter_to_list(
                module_descr, params, id
            )

        if "outputs" in module_descr:
            self._add_grass_module_output_parameter_to_list(
                module_descr, params, id, module_name
            )

        if "flags" in module_descr:
            if "flags" in module_descr:
                params.append("-" + str(module_descr["flags"]))

        if "overwrite" in module_descr and module_descr["overwrite"] is True:
            params.append("--o")

        if "superquiet" in module_descr and module_descr["superquiet"] is True:
            params.append("--qq")

        if "verbose" in module_descr and module_descr["verbose"] is True:
            params.append("--v")

        if (
            "interface-description" in module_descr
            and module_descr["interface-description"] is True
        ):
            params.append("--interface-description")

        # Check for un-allowed characters in the parameter list
        for entry in params:
            if "&" in entry and module_name not in (
                "r.mapcalc",
                "t.rast.mapcalc",
                "t.rast.algebra",
                "t.rast.bandcalc",
            ):
                raise AsyncProcessError(
                    "Character '&' not allowed in "
                    "parameters for %s" % module_name
                )

        if (
            module_name != "importer" and module_name != "exporter"
        ) or params == ["--interface-description"]:
            p = Process(
                exec_type="grass",
                executable=module_name,
                executable_params=params,
                stdin_source=stdin_func,
                id=id,
            )

            self.process_dict[id] = p
            return p

        return None

    def _create_stdin_process(self, module_descr, id):
        """Helper methods to create stdin process.

        Args:
            module_descr (dict): The module description
            id (str): The id of this process in the process chain

        Returns:
            stdin_func(Process): An object of type Process that
                                 contains the module name, the
                                 parameter list and stdin definitions

        """
        if "::" not in module_descr["stdin"]:
            raise AsyncProcessError(
                "The stdin option in id %s misses the ::" % str(id)
            )
        object_id, method = module_descr["stdin"].split("::")
        if "stdout" == method is True:
            stdin_func = self.process_dict[object_id].stdout
        elif "stderr" == method is True:
            stdin_func = self.process_dict[object_id].stderr
        else:
            raise AsyncProcessError(
                "The stdout or stderr flag in id %s is missing" % str(id)
            )
        return stdin_func

    def _create_exec_process(self, module_descr):
        """Analyse a grass process description dict and create a Process
        that is used to execute a common Linux binary.

        Args:
            id (str): The id of this process in the process chain
            module_descr (dict): The module description

        Returns: Process:
            An object of type Process that contains the module name,
            the parameter list and stdout/stdin definitions

        """
        if self.message_logger:
            self.message_logger.info(str(module_descr))

        id = module_descr["id"]

        stdin_func = None
        if "stdin" in module_descr:
            if "::" not in module_descr["stdin"]:
                raise AsyncProcessError(
                    "The stdin option in id %s misses the ::" % str(id)
                )

            object_id, method = module_descr["stdin"].split("::")

            if "stdout" in method:
                stdin_func = self.process_dict[object_id].get_stdout
            elif "stderr" in method:
                stdin_func = self.process_dict[object_id].get_stderr
            else:
                raise AsyncProcessError(
                    "The stdout or stderr flag in "
                    "id %s is missing" % str(id)
                )

        if "exe" not in module_descr:
            raise AsyncProcessError(
                "Missing executable name in module "
                "description of id %s" % str(id)
            )

        executable = module_descr["exe"]

        params = []

        if "params" in module_descr:
            for search_string in module_descr["params"]:
                # Search for file identifiers and generate the temporary file
                # path
                if "$file" in search_string and "::" in search_string:
                    file_id = search_string.split("::")[1]
                    # Generate the temporary file path and store it in the dict
                    if file_id not in self.temporary_pc_files:
                        self.temporary_pc_files[
                            file_id
                        ] = self.generate_temp_file_path()

                    param = "%s" % self.temporary_pc_files[file_id]
                else:
                    param = "%s" % search_string

                params.append(param)

        # Check for un-allowed characters in the parameter list
        for entry in params:
            if "&" in entry:
                raise AsyncProcessError(
                    "Character '&' not supported in process "
                    "description for %s" % executable
                )

        p = Process(
            exec_type="exec",
            executable=executable,
            executable_params=params,
            stdin_source=stdin_func,
            id=id,
        )

        self.process_dict[id] = p

        return p

    def _create_download_process_list(self):
        """This function analysis the process chain import options and creates
        download and import commands.

        Returns:

        """

        downimp_list = []

        if self.message_logger:
            self.message_logger.info(
                "Creating download process " "list for all import definitions"
            )

        sentinel2_entries = []
        for entry in self.import_descr_list:
            if self.message_logger:
                self.message_logger.info(entry)

            check_required_keys_for_download_process_chain(entry)

            # RASTER; VECTOR, FILE
            if (
                entry["import_descr"]["type"].lower() == "raster"
                or entry["import_descr"]["type"].lower() == "vector"
                or entry["import_descr"]["type"].lower() == "file"
            ):
                rvf_downimport_commands = (
                    self._get_raster_vector_file_download_import_command(entry)
                )
                downimp_list.extend(rvf_downimport_commands)

            # POSTGIS
            elif entry["import_descr"]["type"].lower() == "postgis":
                import_command = self._get_postgis_import_command(entry)
                downimp_list.append(import_command)

            # SENTINEL
            elif entry["import_descr"]["type"].lower() == "sentinel2":
                # Old style using Google Big Query. Uncomment/Comment in here
                # to switch.
                # sentinel_commands = self._get_sentinel_import_command(entry)
                # downimp_list.extend(sentinel_commands)
                sentinel2_entries.append(entry)

            # LANDSAT
            elif entry["import_descr"]["type"].lower() == "landsat":
                landsat_commands = self._get_landsat_import_download_commands(
                    entry
                )
                downimp_list.extend(landsat_commands)

            # STAC
            elif entry["import_descr"]["type"].lower() == "stac":
                stac = STAC()
                stac_commands = stac.get_stac_import_download_commands(
                    stac_entry=entry
                )

                downimp_list.extend(stac_commands)

            else:
                raise AsyncProcessError(
                    "Unknown import type specification: %s"
                    % entry["import_descr"]["type"]
                )

        if len(sentinel2_entries) > 0:
            # put all Sentinel-2 downloading together
            sentinel_commands = self._get_sentinel_import_commands(
                sentinel2_entries
            )
            downimp_list.extend(sentinel_commands)
        return downimp_list

    # TODO: remove legacy methods and do no use them in actinia_core
    def _create_module_process_legacy(self, id, module_descr):
        """Analyse a grass process description dict and create a Process
        that is used to execute a GRASS GIS binary.

        Identify the required mapsets from the input definition and stores them
        in a list.

        Args:
            id (str): The id of this process in the process chain
            module_descr (dict): The module description

        Returns: Process:
            An object of type Process that contains the module name,
            the parameter list and stdout/stdin definitions

        """
        self.message_logger.info(str(module_descr))
        parameters = []

        stdin_func = None
        if "stdin" in module_descr:
            if "::" not in module_descr["stdin"]:
                raise AsyncProcessError(
                    "The stdin option in id %s misses the ::" % str(id)
                )

            object_id, method = module_descr["stdin"].split("::")

            if "stdout" == method is True:
                stdin_func = self.process_dict[object_id].stdout

            elif "stderr" == method is True:
                stdin_func = self.process_dict[object_id].stderr
            else:
                raise AsyncProcessError(
                    "The stdout or stderr flag in id %s is missing" % str(id)
                )

        if "module" not in module_descr:
            raise AsyncProcessError(
                "Missing module name in module description of id %s" % str(id)
            )

        module_name = module_descr["module"]

        if "inputs" in module_descr:
            for key in module_descr["inputs"]:
                search_string = str(module_descr["inputs"][key])

                # Search for file identifiers and generate the temporary file
                # path
                if "$file" in search_string and "::" in search_string:
                    file_id = search_string.split("::")[1]
                    # Generate the temporary file path and store it in the dict
                    if file_id not in self.temporary_pc_files:
                        self.temporary_pc_files[
                            file_id
                        ] = self.generate_temp_file_path()

                    param = "%s=%s" % (key, self.temporary_pc_files[file_id])
                else:
                    param = "%s=%s" % (key, module_descr["inputs"][key])
                    # Check for mapset in input name and append it
                    # to the list of required mapsets
                    if "@" in str(module_descr["inputs"][key]):
                        # Mapset names are after an @ symbol
                        # Mapsets in expressions can be detected by replacing
                        # the symbols like *, +, :, /, {, (,},], ... by spaces
                        # and split the string by spaces, searching in each
                        # substring for @

                        symbols = [
                            "*",
                            "+",
                            "-",
                            "/",
                            "%",
                            "$",
                            "!",
                            ":",
                            "(",
                            ")",
                            "{",
                            "}",
                            "&",
                            "?",
                            "#",
                            "=",
                            "^",
                            "~",
                            "<",
                            ">",
                            "\\",
                        ]

                        for symbol in symbols:
                            search_string = search_string.replace(symbol, " ")

                        search_strings = search_string.split(" ")

                        for entry in search_strings:
                            for subentry in entry.split(","):
                                if "@" in subentry:
                                    mapset = subentry.split("@")[1]
                                    if mapset not in self.required_mapsets:
                                        self.required_mapsets.append(mapset)

                parameters.append(param)

        if "outputs" in module_descr:
            for key in module_descr["outputs"]:
                if "name" in module_descr["outputs"][key]:
                    search_string = module_descr["outputs"][key]["name"]

                    # Search for file identifiers and generate the temporary
                    # file path
                    if "$file" in search_string and "::" in search_string:
                        file_id = search_string.split("::")[1]
                        # Generate the temporary file path and store it in the
                        # dict
                        if file_id not in self.temporary_pc_files:
                            self.temporary_pc_files[
                                file_id
                            ] = self.generate_temp_file_path()

                        param = "%s=%s" % (
                            key,
                            self.temporary_pc_files[file_id],
                        )
                    else:
                        param = "%s=%s" % (key, search_string)
                    parameters.append(param)
                    # List the resource for potential export
                    if "export" in module_descr["outputs"][key]:
                        exp = module_descr["outputs"][key]["export"]
                        if "format" not in exp or "type" not in exp:
                            raise AsyncProcessError(
                                "Invalid export parameter in description of "
                                "module <%s>" % module_name
                            )
                        if exp["format"] not in SUPPORTED_EXPORT_FORMATS:
                            raise AsyncProcessError(
                                "Invalid export <format> parameter in "
                                "description of module <%s>" % module_name
                            )
                        if exp["type"] not in [
                            "raster",
                            "vector",
                            "strds",
                            "file",
                            "stvds",
                        ]:
                            raise AsyncProcessError(
                                "Invalid export <type> parameter in "
                                "description of module <%s>" % module_name
                            )
                        self.resource_export_list.append(
                            module_descr["outputs"][key]
                        )

        if "flags" in module_descr:
            if "flags" in module_descr:
                parameters.append("-" + str(module_descr["flags"]))

        if "overwrite" in module_descr and module_descr["overwrite"] is True:
            parameters.append("--o")

        if "superquiet" in module_descr and module_descr["superquiet"] is True:
            parameters.append("--qq")

        if "verbose" in module_descr and module_descr["verbose"] is True:
            parameters.append("--v")

        if (
            "interface-description" in module_descr
            and module_descr["interface-description"] is True
        ):
            parameters.append("--interface-description")

        # Check for un-allowed characters in the parameter list
        for entry in parameters:
            if "&" in entry:
                raise AsyncProcessError(
                    "Character '&' not supported in process "
                    "description for %s" % module_name
                )

        p = Process(
            exec_type="grass",
            executable=module_name,
            executable_params=parameters,
            stdin_source=stdin_func,
            id=id,
        )

        self.process_dict[id] = p

        return p

    # TODO: remove legacy methods and do no use them in actinia_core
    def _create_exec_process_legacy(self, id, module_descr):
        """Analyse a grass process description dict and create a Process
        that is used to execute a common Linux binary.

        Args:
            id (str): The id of this process in the process chain
            module_descr (dict): The module description

        Returns: Process:
            An object of type Process that contains the module name,
            the parameter list and stdout/stdin definitions

        """
        self.message_logger.info(str(module_descr))
        parameters = []

        stdin_func = None
        if "stdin" in module_descr:
            if "::" not in module_descr["stdin"]:
                raise AsyncProcessError(
                    "The stdin option in id %s misses the ::" % str(id)
                )

            object_id, method = module_descr["stdin"].split("::")

            if "stdout" in method:
                stdin_func = self.process_dict[object_id].get_stdout
            elif "stderr" in method:
                stdin_func = self.process_dict[object_id].get_stderr
            else:
                raise AsyncProcessError(
                    "The stdout or stderr flag in id %s is missing" % str(id)
                )

        if "executable" not in module_descr:
            raise AsyncProcessError(
                "Missing executable name in module description of id %s"
                % str(id)
            )

        executable = module_descr["executable"]
        if "parameters" in module_descr:
            for search_string in module_descr["parameters"]:
                # Search for file identifiers and generate the temporary file
                # path
                if "$file" in search_string and "::" in search_string:
                    file_id = search_string.split("::")[1]
                    # Generate the temporary file path and store it in the dict
                    if file_id not in self.temporary_pc_files:
                        self.temporary_pc_files[
                            file_id
                        ] = self.generate_temp_file_path()

                    param = "%s" % self.temporary_pc_files[file_id]
                else:
                    param = "%s" % search_string

                parameters.append(param)

        # Check for un-allowed characters in the parameter list
        for entry in parameters:
            if "&" in entry:
                raise AsyncProcessError(
                    "Character '&' not supported in process "
                    "description for %s" % executable
                )

        p = Process(
            exec_type="exec",
            executable=executable,
            executable_params=parameters,
            stdin_source=stdin_func,
            id=id,
        )

        self.process_dict[id] = p

        return p

    def _add_grass_module_input_parameter_to_list(
        self, module_descr, params, id
    ):
        """Helper method to set the input parameters of a grass module and add
        them to the params list.

        Args:
            module_descr (dict): The module description
            params (list): The list of the grass module inputs parameters with
                           param=value entries (here the input parameter are
                           added)
            id (str): The id of this process in the process chain
        """
        if isinstance(module_descr["inputs"], list) is False:
            raise AsyncProcessError(
                "Inputs in the process chain definition "
                "must be of type list"
            )

        for input in module_descr["inputs"]:
            # Add import description to the import list
            if "import_descr" in input:
                self.import_descr_list.append(input)
            if "value" not in input:
                raise AsyncProcessError(
                    "<value> is missing in input description of process id: %s"
                    % id
                )
            if "param" not in input:
                raise AsyncProcessError(
                    "<param> is missing in input description of process id: %s"
                    % id
                )

            value = input["value"]
            param = input["param"]
            # Search for file identifiers and generate the temporary file path
            if "$file" in value and "::" in value:
                file_id = value.split("::")[1]
                # Generate the temporary file path and store it in the dict
                if file_id not in self.temporary_pc_files:
                    self.temporary_pc_files[
                        file_id
                    ] = self.generate_temp_file_path()
                param = "%s=%s" % (param, self.temporary_pc_files[file_id])
            else:
                param = "%s=%s" % (param, value)
                # Check for mapset in input name and append it,
                # to the list of required mapsets
                if "@" in str(value):
                    # Mapset names are after an @ symbol
                    # Mapsets in expressions can be detected by replacing the
                    # symbols like *, +, :, /, {, (,},], ... by spaces and
                    # split the string by spaces, searching in each substring
                    # for @
                    symbols = [
                        "*",
                        "+",
                        "-",
                        "/",
                        "%",
                        "$",
                        "!",
                        ":",
                        "(",
                        ")",
                        "{",
                        "}",
                        "&",
                        "?",
                        "#",
                        "=",
                        "^",
                        "~",
                        "<",
                        ">",
                        "\\",
                    ]
                    for symbol in symbols:
                        value = value.replace(symbol, " ")
                    values = value.split(" ")
                    for entry in values:
                        for subentry in entry.split(","):
                            if "@" in subentry:
                                mapset = subentry.split("@")[1]
                                if mapset not in self.required_mapsets:
                                    self.required_mapsets.append(mapset)
            params.append(param)

    def _add_grass_module_output_parameter_to_list(
        self, module_descr, params, id, module_name
    ):
        """Helper method to set the output parameters of a grass module and add
        them to the params list. If export is in the output parameter the
        output is appended to the resource_export_list.

        Args:
            module_descr (dict): The module description
            params (list): The list of the grass module parameters with
                           param=value entries (here the output parameter are
                           added)
            id (str): The id of this process in the process chain
            module_name (str): The name of the grass module

        """
        for output in module_descr["outputs"]:
            if "value" not in output:
                raise AsyncProcessError(
                    "<value> is missing in input description of process id: %s"
                    % id
                )
            if "param" not in output:
                raise AsyncProcessError(
                    " <param> is missing in input description of process "
                    "id: %s" % id
                )

            value = output["value"]
            param = output["param"]
            # Check the resource for potential export
            if "export" in output:
                exp = output["export"]
                if "format" not in exp or "type" not in exp:
                    raise AsyncProcessError(
                        "Invalid export parameter in description of module "
                        "<%s>" % module_name
                    )
                if exp["format"] not in SUPPORTED_EXPORT_FORMATS:
                    raise AsyncProcessError(
                        "Invalid export <format> parameter in description of "
                        "module <%s>" % module_name
                    )
                if "PostgreSQL" in exp["format"] and "dbstring" not in exp:
                    raise AsyncProcessError(
                        "The dbstring parameter is missing for PostgreSQL "
                        "export"
                    )
                if exp["type"] not in [
                    "raster",
                    "vector",
                    "strds",
                    "file",
                    "stvds",
                ]:
                    raise AsyncProcessError(
                        "Invalid export <type> parameter in description of "
                        "module <%s>" % module_name
                    )
                if (
                    "file" in exp["type"]
                    and ("$file" in value and "::" in value) is False
                ):
                    raise AsyncProcessError(
                        "The value filed must contain a file identifier "
                        "($file::unique_id) to export a "
                        "file generated from module <%s> as resource."
                        % module_name
                    )
            # Search for file identifiers and generate the temporary file path
            if "$file" in value and "::" in value:
                file_id = value.split("::")[1]
                # Generate the temporary file path and store it in the dict
                if file_id not in self.temporary_pc_files:
                    self.temporary_pc_files[
                        file_id
                    ] = self.generate_temp_file_path()
                # Store the file path in the output description for export
                param = "%s=%s" % (param, self.temporary_pc_files[file_id])
                # Add the temp file path and the new file name with suffix to ,
                # the output dict
                if "export" in output:
                    output["tmp_file"] = self.temporary_pc_files[file_id]
                    output["file_name"] = "%s.%s" % (
                        file_id,
                        output["export"]["format"].lower(),
                    )
            else:
                param = "%s=%s" % (param, value)
            params.append(param)
            # save the output dict in a resource export list
            if "export" in output:
                self.resource_export_list.append(output)

    def _check_if_webhook_exists(self, webhook_url, process_chain, type):
        """
        Helper method to check if the URL of a webhook exists by investigating
        the HTTP header

        Args:
            webhook_url (str): The webhook URL
            process_chain (dict): The process chain
            type (str): The webhook type: 'finished' or 'update'

        Raises:
            This method will raise an AsyncProcessError if the request to the
            webhook url header fails.
        """
        if "auth" in process_chain["webhooks"]:
            self.webhook_auth = process_chain["webhooks"]["auth"]
            # username is expected to be without colon (':')
            resp = requests.head(
                webhook_url,
                auth=HTTPBasicAuth(
                    self.webhook_auth.split(":")[0],
                    ":".join(self.webhook_auth.split(":")[1:]),
                ),
            )
        else:
            resp = requests.head(webhook_url)
        if resp.status_code != 200:
            raise AsyncProcessError(
                "The %s webhook URL %s can not be accessed."
                % (type, webhook_url)
            )

    def _check_and_get_stdout_command(self, module_descr, id):
        """
        Helper method to check the stdout in the module description and add it
        to the output_parser_list.

        Args:
            module_descr (dict): The module description
            id (str): The id of this process in the process chain
        """
        if "id" not in module_descr["stdout"]:
            raise AsyncProcessError(
                "Missing unique *id* in stdout parser description of process "
                "id %s" % str(id)
            )
        if "format" not in module_descr["stdout"]:
            raise AsyncProcessError(
                "Missing *format* in stdout parser description of process "
                "id %s" % str(id)
            )
        if "delimiter" not in module_descr["stdout"]:
            raise AsyncProcessError(
                "Missing *delimiter* in stdout parser description of process "
                "id %s" % str(id)
            )
        self.output_parser_list.append({id: module_descr["stdout"]})

    def _process_chain_to_process_list_legacy(self, process_chain):
        """
        Transform a legacy module chain description into an ordered list of
        process runs

        All input map layer MUST be specified with the mapset they belong to:
        map@mapset

        Args:
            process_chain (dict): The process chain

        Return:
             list
             A list of ordered grass processes

        Process chain input format::

            {                                           # A process chain is a
                                                          dict with entries for
                                                          each module that
                                                          should be run
               Id:{                                     # Id must be numerical
                                                          and indicates the
                                                          process order
                    "module":<module_name>,             # Name of the module to
                                                          run
                    "stdin":<Id::stdout | Id::stderr>   # Use the output of a
                                                          specific module as
                                                          input for this module
                                                        # Id:stdout, Id:stderr
                                                          are available
                    "inputs":{                          # Definition of all
                                                          input parameters as
                                                          key:value pairs
                             <parameter name>:<value>,  # e.g.: value ==
                                                          "raster_name@mapset_
                                                          name" or "degree"
                                                          or 0.0
                             <parameter name>:<value>   # e.g.: value ==
                                                          $file::slope_output_
                                                          file to specify an
                                                          output file
                                                        # that name will be
                                                          automatically
                                                          generated by the API.
                    },
                    "outputs":{                         # Definition of all
                                                          outputs using
                                                          key:value pairs
                        <parameter name>:{
                            "name":<value>,             # Output name e.g.
                                                          "my_aspect_map" or a
                                                          temporary file id
                                                        # definition: $file::id
                                                          eg: $file::aspect_
                                                          output_file
                                                        # This file can be used
                                                          in other module as
                                                          input
                            "export":{                  # Export options, if
                                                          present this map will
                                                          be exported
                                "format":<value>        # Set the export format
                                                          raster=GeoTiff
                                                          (default), vector =
                                                          shape (default)
                                "type":<output type>,   # Output type e.g.:
                                                          raster, vector, file,
                                                          stds
                            }
                        },
                        <parameter name>:{
                            "name":<value>,             # Output name e.g.
                                                          "my_slope_map"
                            "export":{                  # Export options, if
                                                          present this map will
                                                          be exported
                                "format":<value>        # Set the export format
                                                          raster=GeoTiff
                                                          (default), vector =
                                                          shape (default)
                                "type":<output type>,   # Output type e.g.:
                                                          raster, vector, file,
                                                          stds
                            }
                        }
                    },
                    "flags":<flags>,                    # All flags in a string
                                                          e.g.: "ge"
                    "overwrite":<True|False>,           # Set True to overwrite
                                                          existing data
                    "verbose":<True|False>              # Verbosity of the
                                                          module
                },
               Id:{                                     # The id of an
                                                          executable command,
                                                          that is not a grass
                                                          module
                    "executable":<path>,                # The name and path of
                                                          the executable e.g.
                                                          /bin/cp
                    "stdin":<Id::stdout | Id::stderr>   # Use the output of a
                                                          specific module as
                                                          input for this module
                                                        # Id::stdout, Id::
                                                          stderr are available
                    "parameters":[<parameter>,]         # A list of strings
                                                          that represent the
                                                          parameters that may
                                                          contain
                                                        # temporary file
                                                          definitions: $file::
                                                          id  eg: $file::aspect
                                                          _output_file
               },
                ...
            }

        """
        process_list = []

        keys = process_chain.keys()
        int_keys = []
        # Convert the keys to integer to sort correctly

        for k in keys:
            int_keys.append(int(k))

        int_keys = sorted(int_keys)
        # Re-order the process chain by integer sorting index
        for index in int_keys:
            program = process_chain[str(index)]

            if "module" in program:
                process_list.append(
                    self._create_module_process_legacy(str(index), program)
                )
            elif "executable" in program:
                process_list.append(
                    self._create_exec_process_legacy(str(index), program)
                )
            elif "evaluate" in program:
                process_list.append(("python", program["evaluate"]))

        return process_list


def check_required_keys_for_download_process_chain(entry):
    """Helper function to check if the required keys to create the download
    process chain are set.

    Args:
        entry (dict): Entry of the import description list

    Raises:
        If not all required keys are set an AsyncProcessError is raised
    """
    if "import_descr" not in entry:
        raise AsyncProcessError(
            "import_descr specification is required in import definition"
        )
    if "type" not in entry["import_descr"]:
        raise AsyncProcessError(
            "Type specification is required in import definition"
        )
    if "source" not in entry["import_descr"]:
        raise AsyncProcessError(
            "Source specification is required in import definition"
        )
    if entry["import_descr"]["type"] not in [
        "raster",
        "vector",
        "sentinel2",
        "landsat",
        "file",
        "postgis",
        "stac",
    ]:
        raise AsyncProcessError(
            "Unknown type specification: %s" % entry["import_descr"]["type"]
        )


def test_process_chain():
    from pprint import pprint

    elev_in = InputParameter(
        param="elevation",
        value="elev_10m",
        import_descr={
            "source": "https://storage.googleapis.com/graas-geodata/"
            "elev_ned_30m.tif",
            "type": "raster",
        },
    )
    format_in = InputParameter(param="format", value="degree")
    precision_in = InputParameter(param="precision", value="DCELL")
    export = {"type": "raster", "format": "GTiff"}

    slope_out = OutputParameter(
        param="slope", value="elev_10m_slope", export=export
    )

    aspect_out = OutputParameter(
        param="aspect", value="elev_10m_aspect", export=export
    )

    module_1 = GrassModule(
        id="r_slope_aspect_1",
        module="r.slope.aspect",
        inputs=[elev_in, format_in, precision_in],
        outputs=[slope_out, aspect_out],
        flags="a",
        overwrite=True,
        verbose=False,
    )

    file_in = InputParameter(
        param="name",
        value="$file::polygon",
        import_descr={
            "source": "https://storage.googleapis.com/graas-geodata/"
            "brazil_polygon.json",
            "type": "file",
        },
    )

    module_2 = GrassModule(id="importer", module="importer", inputs=[file_in])

    exe_1 = Executable(
        id="cat_1", exe="/bin/cat", params=[], stdin="r_slope_aspect_1::stderr"
    )

    func_in_1 = InputParameter(param="pyfile", value="$file::polygon")

    module_3 = GrassModule(
        id="udf", module="t.rast.aggr_func", inputs=[func_in_1]
    )

    func_in_2 = InputParameter(
        param="pyfile",
        value="$file::polygon",
        import_descr={
            "source": "https://storage.googleapis.com/graas-geodata/"
            "brazil_polygon.json",
            "type": "file",
        },
    )

    module_4 = GrassModule(
        id="udf", module="t.rast.aggr_func", inputs=[func_in_2]
    )

    landsat_import = InputParameter(
        param="map",
        value="ignored",
        import_descr={
            "source": "LT52170762005240COA00",
            "type": "landsat",
            "landsat_atcor": "dos1",
        },
    )

    stdout_parser = StdoutParser(id="stats", format="kv", delimiter="=")

    module_5 = GrassModule(
        id="r_univar_1",
        module="r.univar",
        inputs=[landsat_import],
        stdout=stdout_parser,
    )

    exporter_output = OutputParameter(
        param="map",
        value="LT52170762005240COA00_dos1.1",
        export={"format": "GTiff", "type": "raster"},
    )

    module_export = GrassModule(
        id="exporter_1", module="exporter", outputs=[exporter_output]
    )

    pc = ProcessChainModel(
        version="1",
        list=[
            module_1,
            module_2,
            exe_1,
            module_3,
            module_4,
            module_5,
            module_export,
        ],
    )

    pprint(pc)

    pconv = ProcessChainConverter()
    pl = pconv.process_chain_to_process_list(pc)

    for proc in pl:
        pprint(str(proc))

    pprint(pconv.process_dict)
    pprint(pconv.required_mapsets)
    pprint(pconv.resource_export_list)
    pprint(pconv.import_descr_list)
    pprint(pconv.output_parser_list)
    pprint(pconv.temporary_pc_files)


if __name__ == "__main__":
    test_process_chain()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2023 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
    Setup file for actinia_core.

    This file was generated with PyScaffold 3.0.1.
    PyScaffold helps you to put up the scaffold of your new Python project.
    Learn more under: http://pyscaffold.org/
"""

from setuptools import setup

if __name__ == "__main__":
    setup()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2021 mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Mapset test case
"""
from flask.json import loads as json_load
import uuid
import unittest

try:
    from .test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX

__license__ = "GPLv3"
__author__ = "Julia Haas, Guido Riembauer"
__copyright__ = "Copyright 2021 mundialis GmbH & Co. KG"
__maintainer__ = "mundialis"


class MapsetsTestCase(ActiniaResourceTestCaseBase):
    test_mapsets = [str(uuid.uuid4()), str(uuid.uuid4())]
    test_user = f"test_user_{str(uuid.uuid4())}"
    accessible_datasets = {
        "nc_spm_08": ["PERMANENT"],
        "latlong_wgs84": ["PERMANENT"],
    }
    ref_mapsets = []
    for location in accessible_datasets:
        for mapset in accessible_datasets[location]:
            ref_mapsets.append(f"{location}/{mapset}")

    @classmethod
    def setUpClass(cls):
        super(ActiniaResourceTestCaseBase, cls).setUpClass()

        # Create test_user
        (
            cls.test_user_id,
            cls.test_user_group,
            cls.test_user_auth_header,
        ) = cls.create_user(
            name=cls.test_user,
            role="user",
            accessible_datasets=cls.accessible_datasets,
        )

    def tearDown(self):
        # unlock and delete the test mapsets
        rv = self.server.get(
            URL_PREFIX + "/locations/nc_spm_08/mapsets",
            headers=self.user_auth_header,
        )
        existing_mapsets = json_load(rv.data)["process_results"]
        for mapset in self.test_mapsets:
            if mapset in existing_mapsets:
                rvdellock = self.server.delete(
                    URL_PREFIX
                    + "/locations/nc_spm_08/mapsets/%s/lock" % mapset,
                    headers=self.admin_auth_header,
                )
                print(rvdellock.data.decode())

                rvdel = self.server.delete(
                    URL_PREFIX + "/locations/nc_spm_08/mapsets/%s" % mapset,
                    headers=self.admin_auth_header,
                )
                print(rvdel.data.decode())

    def test_two_locked_mapsets(self):
        # Test correct behaviour if two mapsets are locked
        for mapset in self.test_mapsets:
            self.create_new_mapset(mapset)
            self.server.post(
                URL_PREFIX + "/locations/nc_spm_08/mapsets/%s/lock" % mapset,
                headers=self.root_auth_header,
            )
        rv = self.server.get(
            URL_PREFIX + "/mapsets?status=locked",
            headers=self.root_auth_header,
        )
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )
        rvdata = json_load(rv.data)
        mapset_list = rvdata["locked_mapsets_list"]
        ref_list = ["nc_spm_08/{}".format(ms) for ms in self.test_mapsets]
        for ref_mapset in ref_list:
            self.assertIn(
                ref_mapset,
                mapset_list,
                "%s is not in the list of locked mapsets" % ref_mapset,
            )

        message = rvdata["message"]
        mapsets_no = int(message.split(":")[-1])
        self.assertGreaterEqual(
            mapsets_no, 2, "Number of locked mapsets is smaller than 2"
        )

        print(
            "Currently there are %s locked mapsets:\n %s"
            % (str(mapsets_no), "\n".join(mapset_list))
        )

    def test_user_error(self):
        # Test correct behaviour if user role is not admin
        rv = self.server.get(
            URL_PREFIX + "/mapsets?status=locked",
            headers=self.user_auth_header,
        )
        self.assertEqual(
            rv.status_code, 401, "Status code is not 401: %s" % rv.status_code
        )

    def test_user_own_mapsets(self):
        """Test if user can list available mapsets"""
        rv = self.server.get(
            URL_PREFIX + "/mapsets", headers=self.test_user_auth_header
        )
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        rvdata = json_load(rv.data)
        mapsets = rvdata["available_mapsets"]
        self.assertEqual(
            mapsets,
            self.ref_mapsets,
            "Mapset list is not equal to reference mapset list",
        )

    def test_superadmin_user_mapsets(self):
        """Test if superadmin can list available mapsets from test_user"""
        rv = self.server.get(
            URL_PREFIX + f"/mapsets?user={self.test_user}",
            headers=self.root_auth_header,
        )
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        rvdata = json_load(rv.data)
        mapsets = rvdata["available_mapsets"]
        self.assertEqual(
            mapsets,
            self.ref_mapsets,
            "Mapset list is not equal to reference mapset list",
        )

    def test_user_user_mapsets(self):
        # Test if test_user can list available mapsets from user
        rv = self.server.get(
            URL_PREFIX + f"/mapsets?user={self.test_user}",
            headers=self.user_auth_header,
        )
        self.assertEqual(
            rv.status_code, 401, "Status code is not 401: %s" % rv.status_code
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
STRDS raster map layer management

TODO: Integrate into the ephemeral process chain approach
"""
import tempfile
from actinia_api.swagger2.actinia_core.schemas.strds_raster_management import (
    STRDSRasterListResponseModel,
    STRDSRasterListEntryModel,
)

from actinia_core.processing.actinia_processing.ephemeral.persistent_processing import (
    PersistentProcessing,
)
from actinia_core.core.common.exceptions import AsyncProcessError

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class PersistentRasterSTRDSLister(PersistentProcessing):
    """List all mapsets in a location"""

    def __init__(self, *args):
        PersistentProcessing.__init__(self, *args)
        self.response_model_class = STRDSRasterListResponseModel

    def _execute(self):
        self._setup()

        result_file = tempfile.NamedTemporaryFile(
            dir=self.temp_file_path, delete=True
        )

        pc = {
            "1": {
                "module": "t.rast.list",
                "inputs": {
                    "input": "%s@%s" % (self.map_name, self.mapset_name),
                    "columns": "id,start_time,end_time,north,south,"
                    "east,west,min,max,rows,cols",
                    "separator": "|",
                },
                "outputs": {"output": {"name": result_file.name}},
                "flags": "u",
            }
        }

        if self.rdc.user_data:
            for option in self.rdc.user_data:
                if self.rdc.user_data[option] is not None:
                    pc["1"]["inputs"][option] = self.rdc.user_data[option]

        process_list = self._validate_process_chain(
            skip_permission_check=True, process_chain=pc
        )

        self._create_temp_database(mapsets=self.required_mapsets)
        self._create_grass_environment(
            grass_data_base=self.temp_grass_data_base,
            mapset_name=self.mapset_name,
        )
        self._execute_process_list(process_list)

        result_file.flush()
        result = open(result_file.name, "r").readlines()
        result_file.close()

        output_list = []
        for line in result:
            values = line.strip().split("|")
            output_list.append(
                STRDSRasterListEntryModel(
                    id=values[0],
                    start_time=values[1],
                    end_time=values[2],
                    north=values[3],
                    south=values[4],
                    east=values[5],
                    west=values[6],
                    min=values[7],
                    max=values[8],
                    rows=values[9],
                    cols=values[10],
                )
            )

        self.module_results = output_list


class PersistentRasterSTRDSRegisterer(PersistentProcessing):
    """Register a list of timestamped raster map layers in a STRDS"""

    def __init__(self, *args):
        PersistentProcessing.__init__(self, *args)

    def _execute(self):
        self._setup()

        input_file = tempfile.NamedTemporaryFile(
            dir=self.temp_file_path, delete=True, mode="w"
        )

        for map_entry in self.request_data:
            if (
                "name" not in map_entry
                or "start_time" not in map_entry
                or "end_time" not in map_entry
            ):
                raise AsyncProcessError(
                    "Wrong input format for raster map layer registration"
                )

            line = "%s|%s|%s\n" % (
                map_entry["name"],
                map_entry["start_time"],
                map_entry["end_time"],
            )
            input_file.write(line)
        input_file.flush()
        input_file.seek(0)

        pc = {
            "1": {
                "module": "t.register",
                "inputs": {
                    "input": "%s@%s" % (self.map_name, self.mapset_name),
                    "type": "raster",
                    "separator": "|",
                    "file": input_file.name,
                },
            }
        }

        self._check_lock_target_mapset()
        self._create_temp_database(mapsets=self.required_mapsets)
        process_list = self._validate_process_chain(
            skip_permission_check=True, process_chain=pc
        )
        self._create_grass_environment(
            grass_data_base=self.temp_grass_data_base,
            mapset_name=self.mapset_name,
        )

        self._execute_process_list(process_list)

        input_file.close()


class PersistentRasterSTRDSUnregisterer(PersistentProcessing):
    """Unregister a list of timestamped raster map layers from a STRDS"""

    def __init__(self, *args):
        PersistentProcessing.__init__(self, *args)

    def _execute(self):
        self._setup()

        input_file = tempfile.NamedTemporaryFile(
            dir=self.temp_file_path, delete=True, mode="w"
        )

        for map_name in self.request_data:
            line = "%s\n" % map_name
            input_file.write(line)
        input_file.flush()

        pc = {
            "1": {
                "module": "t.unregister",
                "inputs": {
                    "input": "%s@%s" % (self.map_name, self.mapset_name),
                    "type": "raster",
                    "file": input_file.name,
                },
            }
        }

        process_list = self._validate_process_chain(
            skip_permission_check=True, process_chain=pc
        )
        self._check_lock_target_mapset()
        self._create_temp_database(mapsets=self.required_mapsets)
        self._create_grass_environment(
            grass_data_base=self.temp_grass_data_base,
            mapset_name=self.mapset_name,
        )

        self._execute_process_list(process_list)

        input_file.close()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Compute areal categorical statistics on a raster map layer based on an input
polygon.
"""

import pickle
from flask_restful_swagger_2 import swagger
from flask import jsonify, make_response
from actinia_api.swagger2.actinia_core.apidocs import process_validation

from actinia_core.core.common.app import auth
from actinia_core.core.common.api_logger import log_api_call
from actinia_core.rest.base.endpoint_config import (
    check_endpoint,
    endpoint_decorator,
)
from actinia_core.core.common.redis_interface import enqueue_job
from actinia_core.rest.base.resource_base import ResourceBase
from actinia_core.processing.common.process_validation import start_job

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class AsyncProcessValidationResource(ResourceBase):
    decorators = [log_api_call, auth.login_required]

    @endpoint_decorator()
    @swagger.doc(check_endpoint("post", process_validation.post_doc))
    def post(self, location_name):
        """Validate a process chain asynchronously, check the provided sources
        and the mapsets."""

        rdc = self.preprocess(
            has_json=True, has_xml=True, location_name=location_name
        )

        if rdc:
            rdc.set_storage_model_to_file()
            enqueue_job(self.job_timeout, start_job, rdc)

        html_code, response_model = pickle.loads(self.response_data)
        return make_response(jsonify(response_model), html_code)


class SyncProcessValidationResource(ResourceBase):
    decorators = [log_api_call, auth.login_required]

    @endpoint_decorator()
    @swagger.doc(check_endpoint("post", process_validation.post_doc))
    def post(self, location_name):
        """Validate a process chain synchronously, check the provided sources
        and the mapsets."""

        rdc = self.preprocess(
            has_json=True, has_xml=True, location_name=location_name
        )

        if rdc:
            rdc.set_storage_model_to_file()
            enqueue_job(self.job_timeout, start_job, rdc)
            http_code, response_model = self.wait_until_finish()
        else:
            http_code, response_model = pickle.loads(self.response_data)

        return make_response(jsonify(response_model), http_code)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Raster layer resources
"""

from flask_restful import reqparse
from actinia_core.rest.base.resource_base import ResourceBase

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = (
    "Copyright 2016-2021, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class MapLayerRegionResourceBase(ResourceBase):
    """The base class for map layer (raster, vector) resources that make use of
    region settings
    """

    def create_parser(self):
        """Create the g.region option arguments

        The parameter contain:

            n : for north
            s : for south
            e : for east
            w : for west
            res : resolution of the region
            ewres: east-west resolution
            nsres: north-south resolution
            raster: Raster layer name with mapset to set the region from
            align: Raster layer name with mapset to align the region to
            zoom: Raster layer name with mapset to zoom the region to
            vector: Vector layer name with mapset to set the region from

        Returns:
            The argument parser

        """
        parser = reqparse.RequestParser()
        parser.add_argument(
            "n",
            type=float,
            help="Northern border must be specified as double value",
        )
        parser.add_argument(
            "e",
            type=float,
            help="Eastern border must be specified as double value",
        )
        parser.add_argument(
            "w",
            type=float,
            help="Western border must be specified as double value",
        )
        parser.add_argument(
            "s",
            type=float,
            help="Southern border must be specified as double value",
        )
        parser.add_argument(
            "res",
            type=float,
            help="Resolution must be specified as double value",
        )
        parser.add_argument(
            "nsres",
            type=float,
            help="North-South resolution must be specified as double value",
        )
        parser.add_argument(
            "ewres",
            type=float,
            help="East-West resolution must be specified as double value",
        )
        parser.add_argument(
            "raster",
            type=str,
            help="The raster layer name from which the region should be used, "
            "with mapset information: name@mapset",
        )
        parser.add_argument(
            "align",
            type=str,
            help="The raster layer name from which the region should be "
            "aligned, with mapset information: name@mapset",
        )
        parser.add_argument(
            "zoom",
            type=str,
            help="The raster layer name to which the region should be zoomed, "
            "with mapset information: name@mapset",
        )
        parser.add_argument(
            "vector",
            type=str,
            help="The vector layer name from which the region should be used, "
            "with mapset information: name@mapset",
        )

        return parser

    def create_parser_options(self, args):
        """Parse the request parameters for g.region from parsed arguments
        that were created with the create_parser.

        Args:
            args (dict): The argument dictionary with g.region options

        Returns:
             dict:
             A dict of g.region parameter
        """
        options = {}
        if "n" in args and args["n"] is not None:
            options["n"] = args["n"]
        if "s" in args and args["s"] is not None:
            options["s"] = args["s"]
        if "e" in args and args["e"] is not None:
            options["e"] = args["e"]
        if "w" in args and args["w"] is not None:
            options["w"] = args["w"]

        if "res" in args and args["res"] is not None:
            options["res"] = args["res"]
            if options["res"] < 0:
                return self.get_error_response(
                    message="Resolution can not be negative"
                )

        if "nsres" in args and args["nsres"] is not None:
            options["nsres"] = args["nsres"]
            if options["nsres"] < 0:
                return self.get_error_response(
                    message="North-south resolution can not be negative"
                )

        if "ewres" in args and args["ewres"] is not None:
            options["ewres"] = args["ewres"]
            if options["ewres"] < 0:
                return self.get_error_response(
                    message="East-west resolution can not be negative"
                )

        if "raster" in args and args["raster"] is not None:
            options["raster"] = args["raster"]
            if "@" not in args["raster"]:
                return self.get_error_response(
                    message="The raster layer to set the region from must "
                    "contain the mapset: name@mapset"
                )

        if "align" in args and args["align"] is not None:
            options["align"] = args["align"]
            if "@" not in args["align"]:
                return self.get_error_response(
                    message="The raster layer to align the region from must "
                    "contain the mapset: name@mapset"
                )

        if "zoom" in args and args["zoom"] is not None:
            options["zoom"] = args["zoom"]
            if "@" not in args["zoom"]:
                return self.get_error_response(
                    message="The raster layer to zoom the region to must "
                    "contain the mapset: name@mapset"
                )

        if "vector" in args and args["vector"] is not None:
            options["vector"] = args["vector"]
            if "@" not in args["vector"]:
                return self.get_error_response(
                    message="The vector layer to set the region from must "
                    "contain the mapset: name@mapset"
                )

        return options



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Render base classes
"""

import os
from actinia_core.processing.actinia_processing.ephemeral_processing import (
    EphemeralProcessing,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class EphemeralRendererBase(EphemeralProcessing):
    def __init__(self, *args):
        EphemeralProcessing.__init__(self, *args)

    def _setup_render_environment_and_region(
        self, options, result_file, legacy=True
    ):
        """Setup the render environment and create a g.region
         process chain entry to setup the extent from the options.

        Args:
            options: The parser options that contain n, s, e and w entries for
                     region settings
            result_file: The resulting PNG file name
            legacy: If True use the legacy process chain format for the
                    g.region process definition

        Returns:
            A process chain entry of g.region

        """
        os.putenv("GRASS_RENDER_IMMEDIATE", "png")
        os.putenv("GRASS_RENDER_WIDTH", str(options["width"]))
        os.putenv("GRASS_RENDER_HEIGHT", str(options["height"]))
        os.putenv("GRASS_RENDER_TRANSPARENT", "TRUE")
        os.putenv("GRASS_RENDER_TRUECOLOR", "TRUE")
        os.putenv("GRASS_RENDER_FILE", result_file)
        os.putenv("GRASS_RENDER_FILE_READ", "TRUE")

        if legacy is True:
            pc = {"module": "g.region", "inputs": {}, "flags": "g"}
            if options:
                for key in options:
                    if key in ["n", "s", "e", "w"]:
                        value = options[key]
                        pc["inputs"][key] = value
        else:
            pc = {"id": "2", "module": "g.region", "inputs": [], "flags": "g"}
            if options:
                for key in options:
                    if key in ["n", "s", "e", "w"]:
                        value = options[key]
                        pc["inputs"].append(
                            {"param": key, "value": str(value)}
                        )

        return pc



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Resource storage management

TODO: Tests required
"""

import os
from actinia_core.processing.actinia_processing.ephemeral_processing import (
    EphemeralProcessing,
)
from actinia_core.core.common.process_object import Process
from actinia_core.models.response_models import (
    StorageResponseModel,
    StorageModel,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class ResourceStorageSize(EphemeralProcessing):
    """Compute the download cache size"""

    def __init__(self, *args):
        EphemeralProcessing.__init__(self, *args)
        self.response_model_class = StorageResponseModel
        self.user_resource_storage_path = os.path.join(
            self.config.GRASS_RESOURCE_DIR, self.user_id
        )

    def _execute(self):
        self._setup()

        if not os.path.exists(self.user_resource_storage_path):
            os.mkdir(self.user_resource_storage_path)

        executable = "/usr/bin/du"
        args = ["-sb", self.user_resource_storage_path]

        self._run_process(
            Process(
                exec_type="exec",
                executable=executable,
                id="compute_download_cache_size",
                executable_params=args,
            )
        )

        dc_size = int(self.module_output_log[0]["stdout"].split("\t")[0])
        quota_size = int(self.config.GRASS_RESOURCE_QUOTA * 1024 * 1024 * 1024)

        model = StorageModel(
            used=dc_size,
            free=quota_size - dc_size,
            quota=quota_size,
            free_percent=int(100 * (quota_size - dc_size) / quota_size),
        )
        self.module_results = model

        self.finish_message = "Resource storage size successfully computed"
